@@ -301,11 +301,12 @@ static XS (XS_IRC_get_info)
 {
 	dXSARGS;
 	char *ret;
-	const char *ids[] = {"version", "nick", "channel", "server", "xchatdir"};
+	static const char *ids[] = {"version", "nick", "channel", "server",
+										 "xchatdir", NULL, "network", "host", "topic"};
 	int i = SvIV (ST (0));
 
-	if (i < 5 && i >= 0)
-		XST_mPV (0, xchat_get_info (ph, ids[i]));
+	if (i < 9 && i >= 0 && i != 5)
+		ret = (char *)xchat_get_info (ph, ids[i]);
 	else
 	{
 		switch (i)
@@ -315,13 +316,19 @@ static XS (XS_IRC_get_info)
 				XST_mIV (0, 1);
 			else
 				XST_mIV (0, 0);
-			break;
+			XSRETURN (1);
+			return;
 
 		default:
-			XST_mPV (0, "Error2");
+			ret = "Error2";
 		}
 	}
 
+	if (ret)
+		XST_mPV (0, ret);
+	else
+		XST_mPV (0, "");	/* emulate 1.8.x behaviour */
+
 	XSRETURN (1);
 }
 
@@ -1024,10 +1031,9 @@ static int
 perl_load_file (char *script_name)
 {
 #ifdef WIN32
-	static int have_lib = FALSE;
-	HINSTANCE lib;
+	static HINSTANCE lib = NULL;
 
-	if (!have_lib)
+	if (!lib)
 	{
 		lib = LoadLibrary (PERL_DLL);
 		if (!lib)
@@ -1039,7 +1045,6 @@ perl_load_file (char *script_name)
 							 "Make sure perl's bin directory is in your PATH.");
 			return FALSE;
 		}
-		have_lib = TRUE;
 		FreeLibrary (lib);
 	}
 #endif
