@@ -15,7 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
-#define VERSION "1.0.32"
+#define VERSION "1.0.42"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,6 +74,16 @@ static char sourcedirs[] = {
         "set files [linsert $files 0 $initfile]\n" "}\n" "foreach f $files {\n" "if { [catch { source $f } errMsg] } {\n" "puts \"Tcl plugin\\tError sourcing \\\"$f\\\" ($errMsg)\"\n" "} else {\n" "puts \"Tcl plugin\\tSourced \\\"$f\\\"\"\n" "}\n" "}\n"
 };
 
+static char inlinetcl[] = {
+
+"proc splitsrc { } {\n"
+"uplevel \"scan \\$_src \\\"%\\\\\\[^!\\\\\\]!%\\\\\\[^@\\\\\\]@%s\\\" _nick _ident _host\"\n"
+"}\n"
+"proc ::exit { } {\n"
+"puts \"Using 'exit' is bad\"\n"
+"}\n"
+};
+
 static void Tcl_MyDStringAppend(Tcl_DString * ds, char *string)
 {
     Tcl_DStringAppend(ds, string, strlen(string));
@@ -164,19 +174,38 @@ static char *StrDup(char *string, int *length)
     return result;
 }
 
-static char *myitoa(int value)
+static char *myitoa(long value)
 {
     static char result[32];
-    sprintf(result, "%d", value);
+    sprintf(result, "%ld", value);
     return result;
 }
 
+static xchat_context *atoctx(const char *nptr)
+{
+    int isnum = 1;
+    int x = 0;
+
+    if (!nptr)
+        return NULL;
+
+    while (isnum && nptr[x]) {
+        if (!isdigit(nptr[x++]))
+            isnum--;
+    }
+
+    if (isnum && x)
+        return (xchat_context *) atol(nptr);
+    else
+        return NULL;
+}
 
 static xchat_context *xchat_smart_context(char *arg1, char *arg2)
 {
     char *server, *s, *n;
     xchat_context *result = NULL;
     xchat_context *ctx = NULL;
+    xchat_context *actx = NULL;
     xchat_list *list;
 
     if (arg1 == NULL)
@@ -189,6 +218,8 @@ static xchat_context *xchat_smart_context(char *arg1, char *arg2)
         return result;
     } else {
 
+        actx = atoctx(arg1);
+
         server = xchat_get_info(ph, "server");
 
         list = xchat_list_get(ph, "channels");
@@ -197,25 +228,34 @@ static xchat_context *xchat_smart_context(char *arg1, char *arg2)
 
             while (xchat_list_next(ph, list)) {
 
-                s = xchat_list_str(ph, list, "server");
                 ctx = xchat_list_str(ph, list, "context");
 
-                if (xchat_list_int(ph, list, "type") == 1) {
-                    if (strcasecmp(arg1, s) == 0) {
+                if (actx) {
+                    if (ctx == actx) {
                         result = ctx;
                         break;
                     }
-                    n = xchat_list_str(ph, list, "network");
-                    if (n) {
-                        if (strcasecmp(arg1, n) == 0) {
+                } else {
+
+                    s = xchat_list_str(ph, list, "server");
+
+                    if (xchat_list_int(ph, list, "type") == 1) {
+                        if (strcasecmp(arg1, s) == 0) {
+                            result = ctx;
+                            break;
+                        }
+                        n = xchat_list_str(ph, list, "network");
+                        if (n) {
+                            if (strcasecmp(arg1, n) == 0) {
+                                result = ctx;
+                                break;
+                            }
+                        }
+                    } else {
+                        if ((strcasecmp(server, s) == 0) && (strcasecmp(arg1, xchat_list_str(ph, list, "channel")) == 0)) {
                             result = ctx;
                             break;
                         }
-                    }
-                } else {
-                    if ((strcasecmp(server, s) == 0) && (strcasecmp(arg1, xchat_list_str(ph, list, "channel")) == 0)) {
-                        result = ctx;
-                        break;
                     }
                 }
             }
@@ -596,8 +636,8 @@ static int tcl_timers(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     for (x = 1; x < MAX_TIMERS; x++) {
         if (timers[x].timerid) {
             Tcl_DStringStartSublist(&ds);
-            Tcl_DStringAppendElement(&ds, myitoa(timers[x].timerid));
-            Tcl_DStringAppendElement(&ds, myitoa(timers[x].timestamp - now));
+            Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timerid));
+            Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timestamp - now));
             Tcl_DStringAppendElement(&ds, timers[x].procPtr);
             Tcl_DStringEndSublist(&ds);
         }
@@ -785,7 +825,6 @@ static int tcl_off(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_alias(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     int newentry;
-    int proclen;
     alias *aliasPtr;
     Tcl_HashEntry *entry;
     char *string;
@@ -796,39 +835,52 @@ static int tcl_alias(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     BADARGS(3, 4, " name ?help? {script | procname ?args?}");
 
-    id = (nextprocid++ % INT_MAX) + 1;
-
-    if (SourceInternalProc(id, "_cmd _rest", argv[argc - 1]) == TCL_ERROR) {
-        errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) %s \n", argv[1], errorInfo);
-        return TCL_OK;
-    }
-
     string = StrDup(argv[1], &dummy);
     Tcl_UtfToUpper(string);
 
-    if (argc == 4)
-        help = argv[2];
+    if (strlen(argv[argc - 1])) {
 
-    proclen = strlen(argv[argc - 1]);
+        if (argc == 4)
+            help = argv[2];
+
+        id = (nextprocid++ % INT_MAX) + 1;
+
+        if (SourceInternalProc(id, "_cmd _rest", argv[argc - 1]) == TCL_ERROR) {
+            errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
+            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) %s \n", argv[1], errorInfo);
+            return TCL_OK;
+        }
+
+        entry = Tcl_CreateHashEntry(&aliasTablePtr, string, &newentry);
+        if (newentry) {
+            aliasPtr = (alias *) Tcl_Alloc(sizeof(alias));
+            if (string[0] == '@')
+                aliasPtr->hook = NULL;
+            else
+                aliasPtr->hook = xchat_hook_command(ph, string, XCHAT_PRI_NORM, Command_Alias, help, 0);
+        } else {
+            aliasPtr = Tcl_GetHashValue(entry);
+            DeleteInternalProc(aliasPtr->procPtr);
+            Tcl_Free(aliasPtr->procPtr);
+        }
+
+        aliasPtr->procPtr = StrDup(InternalProcName(id), &dummy);
+
+        Tcl_SetHashValue(entry, aliasPtr);
 
-    entry = Tcl_CreateHashEntry(&aliasTablePtr, string, &newentry);
-    if (newentry) {
-        aliasPtr = (alias *) Tcl_Alloc(sizeof(alias));
-        if (string[0] == '@')
-            aliasPtr->hook = NULL;
-        else
-            aliasPtr->hook = xchat_hook_command(ph, string, XCHAT_PRI_NORM, Command_Alias, help, 0);
     } else {
-        aliasPtr = Tcl_GetHashValue(entry);
-        DeleteInternalProc(aliasPtr->procPtr);
-        Tcl_Free(aliasPtr->procPtr);
+
+        if ((entry = Tcl_FindHashEntry(&aliasTablePtr, string)) != NULL) {
+            aliasPtr = Tcl_GetHashValue(entry);
+            DeleteInternalProc(aliasPtr->procPtr);
+            Tcl_Free(aliasPtr->procPtr);
+            if (aliasPtr->hook)
+                xchat_unhook(ph, aliasPtr->hook);
+            Tcl_Free((char *) aliasPtr);
+            Tcl_DeleteHashEntry(entry);	    
+        }
     }
 
-    aliasPtr->procPtr = StrDup(InternalProcName(id), &dummy);
-
-    Tcl_SetHashValue(entry, aliasPtr);
-
     Tcl_Free(string);
 
     return TCL_OK;
@@ -863,7 +915,7 @@ static int tcl_command(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network|context? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -900,7 +952,7 @@ static int tcl_raw(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network|context? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -937,15 +989,41 @@ static int tcl_getinfo(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
 static int tcl_server(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
-    BADARGS(1, 1, "");
+    xchat_context *origctx, *ctx;
+    BADARGS(1, 2, " ?server|network|context?");
+
+    origctx = xchat_get_context(ph);
+
+    if (argc == 2) {
+        ctx = xchat_smart_context(argv[1], NULL);
+        CHECKCTX(ctx);
+        xchat_set_context(ph, ctx);
+    }
+
     Tcl_AppendResult(irp, xchat_get_info(ph, "server"), NULL);
+
+    xchat_set_context(ph, origctx);
+
     return TCL_OK;
 }
 
 static int tcl_channel(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
-    BADARGS(1, 1, "");
+    xchat_context *origctx, *ctx;
+    BADARGS(1, 2, " ?server|network|context?");
+
+    origctx = xchat_get_context(ph);
+
+    if (argc == 2) {
+        ctx = xchat_smart_context(argv[1], NULL);
+        CHECKCTX(ctx);
+        xchat_set_context(ph, ctx);
+    }
+
     Tcl_AppendResult(irp, xchat_get_info(ph, "channel"), NULL);
+
+    xchat_set_context(ph, origctx);
+
     return TCL_OK;
 }
 
@@ -1029,18 +1107,13 @@ static int tcl_getlist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
                 break;
             case 'i':
                 iattr = xchat_list_int(ph, list, (char *) field);
-                Tcl_DStringAppendElement(&ds, myitoa(iattr));
+                Tcl_DStringAppendElement(&ds, myitoa((long)iattr));
                 break;
             case 'p':
                 sattr = xchat_list_str(ph, list, (char *) field);
                 if (strcmp(field, "context") == 0) {
                     ctx = (xchat_context *) sattr;
-                    xchat_set_context(ph, ctx);
-                    Tcl_DStringStartSublist(&ds);
-                    Tcl_DStringAppendElement(&ds, "ctx");
-                    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "server"));
-                    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "channel"));
-                    Tcl_DStringEndSublist(&ds);
+                    Tcl_DStringAppendElement(&ds, myitoa((long)ctx));
                 } else
                     Tcl_DStringAppendElement(&ds, "*");
                 break;
@@ -1166,7 +1239,7 @@ static int tcl_print(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_context *ctx = NULL;
     char *string = NULL;
 
-    BADARGS(2, 4, " ?server|network? ?#channel|nick? text");
+    BADARGS(2, 4, " ?server|network|context? ?#channel|nick? text");
 
     origctx = xchat_get_context(ph);
 
@@ -1197,18 +1270,10 @@ static int tcl_print(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_setcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *ctx = NULL;
-    int list_argc;
-    char **list_argv;
 
     BADARGS(2, 2, " context");
 
-    if (Tcl_SplitList(interp, argv[1], &list_argc, &list_argv) == TCL_OK) {
-        if ((argc != 3) || strcmp(list_argv[0], "ctx")) {
-            Tcl_AppendResult(irp, "Invalid context", NULL);
-            return TCL_ERROR;
-        }
-        ctx = xchat_find_context(ph, list_argv[1], list_argv[2]);
-    }
+    ctx = xchat_smart_context(argv[1], NULL);
 
     CHECKCTX(ctx);
 
@@ -1219,10 +1284,9 @@ static int tcl_setcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv[
 
 static int tcl_findcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
-    xchat_context *origctx, *ctx = NULL;
-    Tcl_DString ds;
+    xchat_context *ctx = NULL;
 
-    BADARGS(1, 3, " ?server|network? ?channel?");
+    BADARGS(1, 3, " ?server|network|context? ?channel?");
 
     switch (argc) {
     case 1:
@@ -1239,18 +1303,7 @@ static int tcl_findcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv
 
     CHECKCTX(ctx);
 
-    Tcl_DStringInit(&ds);
-
-    origctx = xchat_get_context(ph);
-    xchat_set_context(ph, ctx);
-    Tcl_DStringAppendElement(&ds, "ctx");
-    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "server"));
-    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "channel"));
-    xchat_set_context(ph, origctx);
-
-    Tcl_AppendResult(irp, ds.string, NULL);
-
-    Tcl_DStringFree(&ds);
+    Tcl_AppendResult(irp, myitoa((long)ctx), NULL);
 
     return TCL_OK;
 }
@@ -1258,21 +1311,12 @@ static int tcl_findcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv
 static int tcl_getcontext(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *ctx = NULL;
-    Tcl_DString ds;
 
     BADARGS(1, 1, "");
 
-    Tcl_DStringInit(&ds);
-
     ctx = xchat_get_context(ph);
 
-    Tcl_DStringAppendElement(&ds, "ctx");
-    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "server"));
-    Tcl_DStringAppendElement(&ds, xchat_get_info(ph, "channel"));
-
-    Tcl_AppendResult(irp, ds.string, NULL);
-
-    Tcl_DStringFree(&ds);
+    Tcl_AppendResult(irp, myitoa((long)ctx), NULL);
 
     return TCL_OK;
 }
@@ -1287,10 +1331,10 @@ static int tcl_channels(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     origctx = xchat_get_context(ph);
 
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1362,10 +1406,10 @@ static int tcl_queries(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 
     origctx = xchat_get_context(ph);
 
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1403,7 +1447,7 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
     xchat_list *list;
     Tcl_DString ds;
 
-    BADARGS(1, 3, " ?server|network? ?channel?");
+    BADARGS(1, 3, " ?server|network|context? ?channel?");
 
     origctx = xchat_get_context(ph);
 
@@ -1596,12 +1640,12 @@ static int tcl_dcclist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
                 break;
             }
 
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "size")));
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "resume")));
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "pos")));
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "cps")));
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "address32")));
-            Tcl_DStringAppendElement(&ds, myitoa(xchat_list_int(ph, list, "port")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "size")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "resume")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "pos")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "cps")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "address32")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "port")));
             Tcl_DStringEndSublist(&ds);
         }
         xchat_list_free(ph, list);
@@ -1617,12 +1661,12 @@ static int tcl_dcclist(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_away(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     origctx = xchat_get_context(ph);
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1637,12 +1681,12 @@ static int tcl_away(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_host(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     origctx = xchat_get_context(ph);
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1657,12 +1701,12 @@ static int tcl_host(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_network(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     origctx = xchat_get_context(ph);
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1677,12 +1721,12 @@ static int tcl_network(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network?");
+    BADARGS(1, 2, " ?server|network|context?");
 
     origctx = xchat_get_context(ph);
 
     if (argc == 2) {
-        ctx = xchat_find_context(ph, argv[1], NULL);
+        ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
@@ -1697,7 +1741,7 @@ static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_topic(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     xchat_context *origctx, *ctx = NULL;
-    BADARGS(1, 3, " ?server|network? ?channel?");
+    BADARGS(1, 3, " ?server|network|context? ?channel?");
 
     origctx = xchat_get_context(ph);
 
@@ -1726,7 +1770,7 @@ static int tcl_topic(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 static int tcl_xchat_nickcmp(ClientData cd, Tcl_Interp * irp, int argc, char *argv[])
 {
     BADARGS(3, 3, " string1 string2");
-    Tcl_AppendResult(irp, myitoa(xchat_nickcmp(ph, argv[1], argv[2])), NULL);
+    Tcl_AppendResult(irp, myitoa((long)xchat_nickcmp(ph, argv[1], argv[2])), NULL);
     return TCL_OK;
 }
 
@@ -1892,6 +1936,8 @@ static void Tcl_Plugin_Init()
 
     interp = Tcl_CreateInterp();
 
+    Tcl_FindExecutable(NULL);
+
     Tcl_Init(interp);
 
     nextprocid = 0x1000;
@@ -1951,6 +1997,11 @@ static void Tcl_Plugin_Init()
     if (Tcl_Eval(interp, sourcedirs) == TCL_ERROR) {
         xchat_printf(ph, "Error sourcing internal 'sourcedirs' (%s)\n", Tcl_GetStringResult(interp));
     }
+
+    if (Tcl_Eval(interp, inlinetcl) == TCL_ERROR) {
+        xchat_printf(ph, "Error sourcing internal 'inlinetcl' (%s)\n", Tcl_GetStringResult(interp));
+    }
+
 }
 
 static void Tcl_Plugin_DeInit()
@@ -2030,13 +2081,13 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
 
     if (initialized != 0) {
         banner();
-        xchat_print(ph, "Tcl plugin already loaded.");
+        xchat_print(ph, "Tcl plugin already loaded");
         reinit_tried++;
         return 0;
     }
     initialized = 1;
 
-    *plugin_name = "Tcl plugin";
+    *plugin_name = "Tcl";
     *plugin_desc = "Tcl scripting interface";
     *plugin_version = VERSION;
 
@@ -2051,7 +2102,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     Null_Command_hook = xchat_hook_command(ph, "", XCHAT_PRI_NORM, Null_Command_Alias, "", 0);
 
     banner();
-    xchat_print(ph, "Tcl plugin loaded successfully!\n");
+    xchat_print(ph, "Tcl interface loaded\n");
 
     return 1;                   /* return 1 for success */
 }
@@ -2073,7 +2124,7 @@ int xchat_plugin_deinit()
 
     Tcl_Plugin_DeInit();
 
-    xchat_print(ph, "Tcl plugin unloaded.\n");
+    xchat_print(ph, "Tcl interface unloaded\n");
     initialized = 0;
 
     return 1;
