@@ -19,9 +19,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <string.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -339,6 +336,8 @@ const struct prefs vars[] = {
 	{"away_reason", P_OFFSET (awayreason), TYPE_STR},
 	{"away_show_message", P_OFFINT (show_away_message), TYPE_BOOL},
 	{"away_show_once", P_OFFINT (show_away_once), TYPE_BOOL},
+	{"away_size_max", P_OFFINT (away_size_max), TYPE_INT},
+	{"away_timeout", P_OFFINT (away_timeout), TYPE_INT},
 
 	{"completion_auto", P_OFFINT (nickcompletion), TYPE_BOOL},
 	{"completion_old", P_OFFINT (old_nickcompletion), TYPE_BOOL},
@@ -380,6 +379,10 @@ const struct prefs vars[] = {
 	{"gui_auto_open_dialog", P_OFFINT (autodialog), TYPE_BOOL},
 	{"gui_auto_open_recv", P_OFFINT (autoopendccrecvwindow), TYPE_BOOL},
 	{"gui_auto_open_send", P_OFFINT (autoopendccsendwindow), TYPE_BOOL},
+	{"gui_dialog_height", P_OFFINT (dialog_height), TYPE_INT},
+	{"gui_dialog_left", P_OFFINT (dialog_left), TYPE_INT},
+	{"gui_dialog_top", P_OFFINT (dialog_top), TYPE_INT},
+	{"gui_dialog_width", P_OFFINT (dialog_width), TYPE_INT},
 	{"gui_hide_menu", P_OFFINT (hidemenu), TYPE_BOOL},
 	{"gui_input_style", P_OFFINT (style_inputbox), TYPE_BOOL},
 	{"gui_lagometer", P_OFFINT (lagometer), TYPE_INT},
@@ -408,6 +411,9 @@ const struct prefs vars[] = {
 	{"hebrew", P_OFFINT (hebrew), TYPE_BOOL},
 #endif
 
+#ifdef WIN32
+	{"identd", P_OFFINT (identd), TYPE_BOOL},
+#endif
 	{"input_beep_chans", P_OFFINT (beepchans), TYPE_BOOL},
 	{"input_beep_hilight", P_OFFINT (beephilight), TYPE_BOOL},
 	{"input_beep_msg", P_OFFINT (beepmsg), TYPE_BOOL},
@@ -510,11 +516,13 @@ load_config (void)
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 
 	/* put in default values, anything left out is automatically zero */
+	prefs.away_timeout = 60;
+	prefs.away_size_max = 300;
 	prefs.timestamp_logs = 1;
 	prefs.truncchans = 20;
 	prefs.autoresume = 1;
 	prefs.show_away_once = 1;
-	prefs.show_away_message = 1;
+	/*prefs.show_away_message = 1;*/
 	prefs.indent_nicks = 1;
 	prefs.thin_separator = 1;
 	/*prefs.tabs_position = 1;*/ /* 0 = bottom */
@@ -530,7 +538,6 @@ load_config (void)
 	prefs.use_server_tab = 1;
 	/*prefs.windows_as_tabs = 1;*/
 	prefs.privmsgtab = 1;
-	prefs.nickcompletion = 1;
 	prefs.style_inputbox = 1;
 	prefs.slist_select = 9;
 	prefs.nu_color = 4;
@@ -538,6 +545,8 @@ load_config (void)
 	prefs.max_lines = 300;
 	prefs.mainwindow_width = 640;
 	prefs.mainwindow_height = 400;
+	prefs.dialog_width = 500;
+	prefs.dialog_height = 256;
 	prefs.dcctimeout = 180;
 	prefs.dccstalltimeout = 60;
 	prefs.notify_timeout = 15;
@@ -563,10 +572,14 @@ load_config (void)
 	prefs.perc_color = 1;
 	prefs.dcc_send_fillspaces = 1;
 	prefs.mainwindow_save = 1;
+	prefs.bantype = 2;
+#ifdef WIN32
+	prefs.identd = 1;
+#endif
 	strcpy (prefs.stamp_format, "[%H:%M] ");
 	strcpy (prefs.timestamp_log_format, "%b %d %H:%M:%S ");
 	strcpy (prefs.logmask, "%n-%c.log");
-	strcpy (prefs.nick_suffix, ":");
+	strcpy (prefs.nick_suffix, ",");
 	strcpy (prefs.cmdchar, "/");
 	strcpy (prefs.nick1, username);
 	strcpy (prefs.nick2, username);
@@ -576,18 +589,18 @@ load_config (void)
 	strcpy (prefs.realname, realname);
 	strcpy (prefs.username, username);
 #ifdef WIN32
-	strcpy (prefs.sounddir, "./sound");
+	strcpy (prefs.sounddir, "./sounds");
 	if (strcmp (get_xdir (), "./config") != 0)
-		sprintf (prefs.dccdir, "%s\\download", get_xdir ());
+		sprintf (prefs.dccdir, "%s\\downloads", get_xdir ());
 	else
-		strcpy (prefs.dccdir, "./download");
+		strcpy (prefs.dccdir, "./downloads");
 #else
-	sprintf (prefs.sounddir, "%s/sound", g_get_home_dir ());
-	sprintf (prefs.dccdir, "%s/dcc", g_get_home_dir ());
+	sprintf (prefs.sounddir, "%s/sounds", get_xdir ());
+	sprintf (prefs.dccdir, "%s/downloads", get_xdir ());
 #endif
-	strcpy (prefs.doubleclickuser, "QUOTE WHOIS %s");
+	strcpy (prefs.doubleclickuser, "QUOTE WHOIS %s %s");
 	strcpy (prefs.awayreason, _("I'm busy"));
-	strcpy (prefs.quitreason, _("Client exiting"));
+	strcpy (prefs.quitreason, _("Leaving"));
 	strcpy (prefs.partreason, prefs.quitreason);
 	strcpy (prefs.font_normal, DEF_FONT);
 	strcpy (prefs.soundcmd, "esdplay");
