@@ -22,6 +22,8 @@
  * Jim Seymour (jseymour@LinxNet.com)
  */
 
+/* we only use 32 bits, but without this define, you get only 31! */
+#define _FILE_OFFSET_BITS 64
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -110,10 +112,11 @@ static void
 dcc_calc_cps (struct DCC *dcc)
 {
 	GTimeVal now;
-	int pos, posdiff, oldcps;
+	int oldcps;
 	double timediff, startdiff;
 	int glob_throttle_bit, wasthrottled;
 	int *cpssum, glob_limit;
+	unsigned int pos, posdiff;
 
 #ifdef WIN32
 	g_get_current_time_win32 (&now);
@@ -125,7 +128,8 @@ dcc_calc_cps (struct DCC *dcc)
 		between pos and ack */
 	if (dcc->type == TYPE_SEND)
 	{
-		pos = (dcc->pos + dcc->ack) / 2;
+		/* carefull to avoid 32bit overflow */
+		pos = dcc->pos - ((dcc->pos - dcc->ack) / 2);
 		glob_throttle_bit = 0x1;
 		cpssum = &dcc_sendcpssum;
 		glob_limit = prefs.dcc_global_max_send_cps;
@@ -343,8 +347,10 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 			/* completed file to the completed directory */
 			if(dcc->type == TYPE_RECV)
 			{			
+				/* mgl: change this to use destfile_fs for correctness and to */
+				/* handle the case where dccwithnick is set */
 				download_move_to_completed_dir(prefs.dccdir, prefs.dcc_completed_dir, 
-					dcc->file, prefs.dccpermissions);
+					dcc->destfile_fs, prefs.dccpermissions);
 			}
 
 		}
@@ -404,12 +410,12 @@ dcc_abort (session *sess, struct DCC *dcc)
 								 NULL, 0);
 				break;
 			case TYPE_SEND:
-				EMIT_SIGNAL (XP_TE_DCCSENDABORT, sess, file_part (dcc->file),
-								 dcc->nick, NULL, NULL, 0);
+				EMIT_SIGNAL (XP_TE_DCCSENDABORT, sess, dcc->nick,
+								 file_part (dcc->file), NULL, NULL, 0);
 				break;
 			case TYPE_RECV:
-				EMIT_SIGNAL (XP_TE_DCCRECVABORT, sess, dcc->file,
-								 dcc->nick, NULL, NULL, 0);
+				EMIT_SIGNAL (XP_TE_DCCRECVABORT, sess, dcc->nick,
+								 dcc->file, NULL, NULL, 0);
 			}
 			break;
 		default:
@@ -552,10 +558,10 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		po = strchr (line + 8, '\001');
 		if (po)
 			po[0] = 0;
-		inbound_action (sess, tbuf, dcc->serv->nick, dcc->nick, line + 8, FALSE);
+		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE);
 	} else
 	{
-		inbound_privmsg (dcc->serv, tbuf, dcc->nick, "", line);
+		inbound_privmsg (dcc->serv, dcc->nick, "", line);
 	}
 	if (utf)
 		g_free (utf);
@@ -620,7 +626,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 					dead = dcc_chat_line (dcc, dcc->dccchat->linebuf, tbuf);
 				}
 
-				if (dead) /* the dcc has been closed, don't use (DCC *)! */
+				if (dead || !dcc->dccchat) /* the dcc has been closed, don't use (DCC *)! */
 					return TRUE;
 
 				dcc->pos += dcc->dccchat->pos;
@@ -1212,6 +1218,12 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file, int maxcps)
 
 	if (stat (file_fs, &st) != -1)
 	{
+		if (sizeof (st.st_size) > 4 && st.st_size > 4294967295U)
+		{
+			PrintText (sess, "Cannot send files larger than 4 GB.\n");
+			goto noaxs;
+		}
+
 		if (*file_part (file_fs) && !S_ISDIR (st.st_mode))
 		{
 			if (st.st_size > 0)
@@ -1264,8 +1276,9 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file, int maxcps)
 			}
 		}
 	}
-	g_free (file_fs);
 	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
+noaxs:
+	g_free (file_fs);
 	dcc_close (dcc, 0, TRUE);
 }
 
@@ -1460,7 +1473,7 @@ dcc_malformed (struct session *sess, char *nick, char *data)
 	EMIT_SIGNAL (XP_TE_MALFORMED, sess, nick, data, NULL, NULL, 0);
 }
 
-void
+int
 dcc_resume (struct DCC *dcc)
 {
 	char tbuf[500];
@@ -1477,7 +1490,10 @@ dcc_resume (struct DCC *dcc)
  			sprintf (tbuf + strlen (tbuf), " %d", dcc->pasvid);
 
 		dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
+		return 1;
 	}
+
+	return 0;
 }
 
 void
@@ -1498,7 +1514,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 		if (port == 0)
 			pasvid = atoi (word[9]);
 
-		if (!addr || (port < 1024 && port != 0)
+		if (!addr /*|| (port < 1024 && port != 0)*/
 			|| port > 0xffff || (port == 0 && pasvid == 0))
 		{
 			dcc_malformed (sess, nick, word_eol[4] + 2);
