@@ -19,10 +19,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -253,6 +249,8 @@ ignore_read_next_entry (char *my_cfg, struct ignore *ignore)
 	if (my_cfg)
 	{
 		my_cfg = cfg_get_str (my_cfg, "mask", tbuf);
+		if (!my_cfg)
+			return NULL;
 		ignore->mask = strdup (tbuf);
 	}
 	if (my_cfg)
