@@ -65,6 +65,8 @@ clear_channel (session *sess)
 {
 	strcpy (sess->waitchannel, sess->channel);
 	sess->channel[0] = 0;
+	sess->doing_who = FALSE;
+	sess->done_away_check = FALSE;
 
 	log_close (sess);
 
@@ -131,7 +133,7 @@ find_session_from_nick (char *nick, server *serv)
 }
 
 void
-inbound_privmsg (server *serv, char *tbuf, char *from, char *ip, char *text)
+inbound_privmsg (server *serv, char *from, char *ip, char *text)
 {
 	session *sess;
 
@@ -155,18 +157,24 @@ inbound_privmsg (server *serv, char *tbuf, char *from, char *ip, char *text)
 			if (prefs.logging && sess->logfd != -1 &&
 				(!sess->topic || strcmp(sess->topic, ip)))
 			{
-				snprintf (tbuf, 2048, "[%s has address %s]\n", from, ip);
+				char tbuf[1024];
+				snprintf (tbuf, sizeof (tbuf), "[%s has address %s]\n", from, ip);
 				write (sess->logfd, tbuf, strlen (tbuf));
 			}
 			set_topic (sess, ip);
 		}
-		inbound_chanmsg (serv, tbuf, from, from, text, FALSE);
+		inbound_chanmsg (serv, NULL, from, text, FALSE);
 		return;
 	}
+
 	sess = find_session_from_nick (from, serv);
 	if (!sess)
 		sess = serv->front_session;
-	EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
+
+	if (sess->type == SESS_DIALOG)
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
+	else
+		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
 }
 
 static int
@@ -219,8 +227,7 @@ is_hilight (char *text, session *sess, server *serv)
 }
 
 void
-inbound_action (session *sess, char *tbuf, char *chan, char *from, char *text,
-					 int fromme)
+inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 {
 	session *def = sess;
 	server *serv = sess->server;
@@ -249,8 +256,15 @@ inbound_action (session *sess, char *tbuf, char *chan, char *from, char *text,
 
 	if (sess != current_tab)
 	{
-		sess->msg_said = TRUE;
-		sess->new_data = FALSE;
+		if (fromme)
+		{
+			sess->msg_said = FALSE;
+			sess->new_data = TRUE;
+		} else
+		{
+			sess->msg_said = TRUE;
+			sess->new_data = FALSE;
+		}
 	}
 
 	if (!fromme)
@@ -267,7 +281,8 @@ inbound_action (session *sess, char *tbuf, char *chan, char *from, char *text,
 
 	if (prefs.colorednicks)
 	{
-		sprintf (tbuf, "\003%d%s", color_of (from), from);
+		char tbuf[NICKLEN + 4];
+		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
 		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, NULL, NULL, 0);
 	} else
 	{
@@ -276,17 +291,22 @@ inbound_action (session *sess, char *tbuf, char *chan, char *from, char *text,
 }
 
 void
-inbound_chanmsg (server *serv, char *tbuf, char *chan, char *from, char *text,
-					  char fromme)
+inbound_chanmsg (server *serv, char *chan, char *from, char *text, char fromme)
 {
 	struct User *user;
 	session *sess;
 	int hilight = FALSE;
 	char nickchar[2] = "\000";
 
-	sess = find_channel (serv, chan);
-	if (!sess && !is_channel (serv, chan))
-		sess = find_dialog (serv, chan);
+	if (chan)
+	{
+		sess = find_channel (serv, chan);
+		if (!sess && !is_channel (serv, chan))
+			sess = find_dialog (serv, chan);
+	} else
+	{
+		sess = find_dialog (serv, from);
+	}
 	if (!sess)
 		return;
 
@@ -322,12 +342,13 @@ inbound_chanmsg (server *serv, char *tbuf, char *chan, char *from, char *text,
 			fe_beep ();
 	}
 	if (sess->type == SESS_DIALOG)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, nickchar, NULL, 0);
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
 	else if (hilight)
 		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, NULL, 0);
 	else if (prefs.colorednicks)
 	{
-		sprintf (tbuf, "\003%d%s", color_of (from), from);
+		char tbuf[NICKLEN + 4];
+		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
 		EMIT_SIGNAL (XP_TE_CHANMSG, sess, tbuf, text, nickchar, NULL, 0);
 	}
 	else
@@ -352,7 +373,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (change_nick (sess, nick, newnick))
+			if (change_nick (sess, nick, newnick) || (me && sess->type == SESS_SERVER))
 			{
 				if (!quiet)
 				{
@@ -364,7 +385,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 										 NULL, 0);
 				}
 			}
-			if (!serv->p_cmp (sess->channel, nick))
+			if (sess->type == SESS_DIALOG && !serv->p_cmp (sess->channel, nick))
 			{
 				safe_strcpy (sess->channel, newnick, CHANLEN);
 				fe_set_channel (sess);
@@ -687,10 +708,11 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 }
 
 void
-inbound_ping_reply (session *sess, char *outbuf, char *timestring, char *from)
+inbound_ping_reply (session *sess, char *timestring, char *from)
 {
 	unsigned long tim, nowtim, dif;
 	int lag = 0;
+	char outbuf[64];
 
 	if (strncmp (timestring, "LAG", 3) == 0)
 	{
@@ -707,7 +729,7 @@ inbound_ping_reply (session *sess, char *outbuf, char *timestring, char *from)
 	if (lag)
 	{
 		sess->server->lag_sent = 0;
-		sprintf (outbuf, "%ld.%ld", dif / 100000, (dif / 10000) % 100);
+		snprintf (outbuf, sizeof (outbuf), "%ld.%ld", dif / 100000, (dif / 10000) % 100);
 		fe_set_lag (sess->server, (int)((float)atof (outbuf)));
 		return;
 	}
@@ -720,7 +742,7 @@ inbound_ping_reply (session *sess, char *outbuf, char *timestring, char *from)
 			EMIT_SIGNAL (XP_TE_PINGREP, sess, from, "?", NULL, NULL, 0);
 	} else
 	{
-		sprintf (outbuf, "%ld.%ld%ld", dif / 1000000, (dif / 100000) % 10, dif % 10);
+		snprintf (outbuf, sizeof (outbuf), "%ld.%ld%ld", dif / 1000000, (dif / 100000) % 10, dif % 10);
 		EMIT_SIGNAL (XP_TE_PINGREP, sess, from, outbuf, NULL, NULL, 0);
 	}
 }
@@ -741,8 +763,7 @@ find_session_from_type (int type, server *serv)
 }
 
 void
-inbound_notice (server *serv, char *outbuf, char *to, char *nick, char *msg,
-					 char *ip)
+inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
 {
 	char *po,*ptr=to;
 	session *sess = 0;
@@ -810,7 +831,7 @@ inbound_notice (server *serv, char *outbuf, char *to, char *nick, char *msg,
 		msg++;
 		if (!strncmp (msg, "PING", 4))
 		{
-			inbound_ping_reply (sess, outbuf, msg + 5, nick);
+			inbound_ping_reply (sess, msg + 5, nick);
 			return;
 		}
 	}
@@ -831,6 +852,7 @@ inbound_away (server *serv, char *nick, char *msg)
 {
 	struct away_msg *away = find_away_message (serv, nick);
 	session *sess = NULL;
+	GSList *list;
 
 	if (away && !strcmp (msg, away->message))	/* Seen the msg before? */
 	{
@@ -847,6 +869,15 @@ inbound_away (server *serv, char *nick, char *msg)
 		sess = serv->server_session;
 
 	EMIT_SIGNAL (XP_TE_WHOIS5, sess, nick, msg, NULL, NULL, 0);
+
+	list = sess_list;
+	while (list)
+	{
+		sess = list->data;
+		if (sess->server == serv)
+			userlist_set_away (sess, nick, TRUE);
+		list = list->next;
+	}
 }
 
 int
@@ -932,15 +963,16 @@ inbound_next_nick (session *sess, char *nick)
 }
 
 void
-do_dns (session *sess, char *tbuf, char *nick, char *host)
+do_dns (session *sess, char *nick, char *host)
 {
 	char *po;
+	char tbuf[1024];
 
 	po = strrchr (host, '@');
 	if (po)
 		host = po + 1;
 	EMIT_SIGNAL (XP_TE_RESOLVINGUSER, sess, nick, host, NULL, NULL, 0);
-	snprintf (tbuf, 2048, "exec -d %s %s", prefs.dnsprogram, host);
+	snprintf (tbuf, sizeof (tbuf), "exec -d %s %s", prefs.dnsprogram, host);
 	handle_command (sess, tbuf, FALSE);
 }
 
@@ -980,12 +1012,30 @@ inbound_login_start (session *sess, char *nick, char *servname)
 	}
 }
 
+static void
+inbound_set_all_away_status (server *serv, char *nick, unsigned int status)
+{
+	GSList *list;
+	session *sess;
+
+	list = sess_list;
+	while (list)
+	{
+		sess = list->data;
+		if (sess->server == serv)
+			userlist_set_away (sess, nick, status);
+		list = list->next;
+	}
+}
+
 void
 inbound_uaway (server *serv)
 {
 	serv->is_away = TRUE;
 	serv->away_time = time (NULL);
 	fe_set_away (serv);
+
+	inbound_set_all_away_status (serv, serv->nick, 1);
 }
 
 void
@@ -994,6 +1044,8 @@ inbound_uback (server *serv)
 	serv->is_away = FALSE;
 	serv->reconnect_away = FALSE;
 	fe_set_away (serv);
+
+	inbound_set_all_away_status (serv, serv->nick, 0);
 }
 
 void
@@ -1011,27 +1063,52 @@ inbound_foundip (session *sess, char *ip)
 	}
 }
 
+void
+inbound_user_info_start (session *sess, char *nick)
+{
+	/* set away to FALSE now, 301 may turn it back on */
+	inbound_set_all_away_status (sess->server, nick, 0);
+}
+
 int
-inbound_user_info (session *sess, char *outbuf, char *chan, char *user,
-						 char *host, char *servname, char *nick, char *realname)
+inbound_user_info (session *sess, char *chan, char *user, char *host,
+						 char *servname, char *nick, char *realname,
+						 unsigned int away)
 {
 	server *serv = sess->server;
 	session *who_sess;
+	char *uhost;
 
 	who_sess = find_channel (serv, chan);
 	if (who_sess)
 	{
-		snprintf (outbuf, 2048, "%s@%s", user, host);
-		if (!userlist_add_hostname (who_sess, nick, outbuf, realname, servname))
+		if (user && host)
 		{
-			if (!who_sess->doing_who)
-				return 0;
+			uhost = malloc (strlen (user) + strlen (host) + 2);
+			sprintf (uhost, "%s@%s", user, host);
+			if (!userlist_add_hostname (who_sess, nick, uhost, realname, servname, away))
+			{
+				if (!who_sess->doing_who)
+				{
+					free (uhost);
+					return 0;
+				}
+			}
+			free (uhost);
+		} else
+		{
+			if (!userlist_add_hostname (who_sess, nick, NULL, realname, servname, away))
+			{
+				if (!who_sess->doing_who)
+					return 0;
+			}
 		}
 	} else
 	{
-		if (!serv->doing_who)
+		if (!serv->doing_dns)
 			return 0;
-		do_dns (sess, outbuf, nick, host);
+		if (nick && host)
+			do_dns (sess, nick, host);
 	}
 	return 1;
 }
