@@ -18,10 +18,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -103,7 +99,7 @@ notify_load (void)
 	{
 		while (waitline (fh, buf, sizeof buf) != -1)
 		{
-			if (*buf != '#')
+			if (buf[0] != '#' && buf[0] != 0)
 				notify_adduser (buf);
 		}
 		close (fh);
@@ -128,7 +124,7 @@ notify_find (server *serv, char *nick)
 			continue;
 		}
 
-		if (!rfc_casecmp (notify->name, nick))
+		if (!serv->p_cmp (notify->name, nick))
 			return servnot;
 
 		list = list->next;
@@ -210,7 +206,7 @@ notify_watch (server * serv, char *nick, int add)
 {
 	char tbuf[256];
 
-	snprintf (tbuf, sizeof (tbuf), "WATCH +%s\r\n", nick);
+	snprintf (tbuf, sizeof (tbuf), "WATCH +%s", nick);
 	if (!add)
 		tbuf[6] = '-';
 	serv->p_raw (serv, tbuf);
@@ -247,7 +243,6 @@ notify_flush_watches (server * serv, GSList *from, GSList *end)
 		strcat (tbuf, notify->name);
 		list = list->next;
 	}
-	strcat (tbuf, "\r\n");
 	serv->p_raw (serv, tbuf);
 }
 
@@ -306,7 +301,7 @@ notify_markonline (server *serv, char *word[])
 		seen = FALSE;
 		while (*word[i])
 		{
-			if (!rfc_casecmp (notify->name, word[i]))
+			if (!serv->p_cmp (notify->name, word[i]))
 			{
 				seen = TRUE;
 				notify_announce_online (serv, servnot, notify->name);
@@ -366,7 +361,6 @@ notify_checklist (void)
 	if (i)
 	{
 		GSList *list = serv_list;
-		strcat (outbuf, "\r\n");
 		while (list)
 		{
 			serv = (struct server *) list->data;
@@ -469,7 +463,7 @@ notify_isnotify (struct session *sess, char *name)
 	while (list)
 	{
 		notify = (struct notify *) list->data;
-		if (!rfc_casecmp (notify->name, name))
+		if (!sess->server->p_cmp (notify->name, name))
 		{
 			servnot = notify_find_server_entry (notify, sess->server);
 			if (servnot && servnot->ison)
