@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#define _GNU_SOURCE	/* for memrchr */
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -975,6 +976,10 @@ cmd_discon (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
+#ifdef WIN32
+	PrintText (sess, "DNS is not implemented in Windows.\n");
+	return TRUE;
+#else
 	char *nick = word[2];
 	struct User *user;
 
@@ -985,11 +990,11 @@ cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			user = find_name (sess, nick);
 			if (user && user->hostname)
 			{
-				do_dns (sess, tbuf, user->nick, user->hostname);
+				do_dns (sess, user->nick, user->hostname);
 			} else
 			{
 				sess->server->p_get_ip (sess->server, nick);
-				sess->server->doing_who = TRUE;
+				sess->server->doing_dns = TRUE;
 			}
 		} else
 		{
@@ -999,6 +1004,7 @@ cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		return TRUE;
 	}
 	return FALSE;
+#endif
 }
 
 static int
@@ -1224,6 +1230,19 @@ norm:			nbuf[j] = buf[i];
 	free (nbuf);
 }
 
+#ifndef HAVE_MEMRCHR
+static void *
+memrchr (const void *block, int c, size_t size)
+{
+	unsigned char *p;
+
+	for (p = (unsigned char *)block + size; p != block; p--)
+		if (*p == c)
+			return p;
+	return 0;
+}
+#endif
+
 static gboolean
 exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 {
@@ -1266,8 +1285,8 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 	}
 	len += rd;
 	buf[len] = '\0';
-	
-	rest = strrchr(buf, '\n');
+
+	rest = memrchr(buf, '\n', len);
 	if (rest)
 		rest++;
 	else
@@ -1523,6 +1542,26 @@ cmd_getstr (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (!strcasecmp (word[2], "HIDE"))
+	{
+		fe_ctrl_gui (sess, 0);
+	} else if (!strcasecmp (word[2], "SHOW"))
+	{
+		fe_ctrl_gui (sess, 1);
+	} else if (!strcasecmp (word[2], "FOCUS"))
+	{
+		fe_ctrl_gui (sess, 2);
+	} else
+	{
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1748,6 +1787,13 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_killall (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	xchat_exit();
+	return 2;
+}
+
 static int
 cmd_lagcheck (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1859,13 +1905,18 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (!(*act))
 		return FALSE;
 
+	if (sess->type == SESS_SERVER)
+	{
+		notj_msg (sess);
+		return TRUE;
+	}
+
 	sprintf (tbuf, "\001ACTION %s\001\r", act);
 	/* first try through DCC CHAT */
 	if (dcc_write_chat (sess->channel, tbuf))
 	{
 		/* print it to screen */
-		inbound_action (sess, tbuf, sess->channel, sess->server->nick,
-							 act, TRUE);
+		inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
 	} else
 	{
 		/* DCC CHAT failed, try through server */
@@ -1873,8 +1924,7 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			sess->server->p_action (sess->server, sess->channel, act);
 			/* print it to screen */
-			inbound_action (sess, tbuf, sess->channel, sess->server->nick,
-								 act, TRUE);
+			inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
 		} else
 		{
 			notc_msg (sess);
@@ -1952,8 +2002,8 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			if (!newsess)
 				newsess = find_channel (sess->server, nick);
 			if (newsess)
-				inbound_chanmsg (newsess->server, tbuf, newsess->channel,
-						newsess->server->nick, msg, TRUE);
+				inbound_chanmsg (newsess->server, newsess->channel,
+									  newsess->server->nick, msg, TRUE);
 			else
 				EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
 
@@ -2393,8 +2443,8 @@ userlist_cb (struct User *user, session *sess)
 	else
 		lt = time (0) - user->lasttalk;
 	PrintTextf (sess,
-				"\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d lt\0033=\017%d\n",
-				user->nick, user->hostname, user->op, user->voice, lt);
+				"\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d away=%u lt\0033=\017%d\n",
+				user->nick, user->hostname, user->op, user->voice, user->away, lt);
 
 	return TRUE;
 }
@@ -2470,7 +2520,7 @@ cmd_wallchan (struct session *sess, char *tbuf, char *word[],
 			sess = list->data;
 			if (sess->type == SESS_CHANNEL)
 			{
-				inbound_chanmsg (sess->server, tbuf, sess->channel,
+				inbound_chanmsg (sess->server, sess->channel,
 									  sess->server->nick, word_eol[2], TRUE);
 				sess->server->p_message (sess->server, sess->channel, word_eol[2]);
 			}
@@ -2578,6 +2628,7 @@ const struct commands xc_cmds[] = {
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
 	{"GETINT", cmd_getint, 0, 0, "GETINT <default> <command> <prompt>"},
 	{"GETSTR", cmd_getstr, 0, 0, "GETSTR <default> <command> <prompt>"},
+	{"GUI", cmd_gui, 0, 0, "GUI [SHOW|HIDE|FOCUS]"},
 	{"HELP", cmd_help, 0, 0, 0},
 	{"HOP", cmd_hop, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
@@ -2595,6 +2646,7 @@ const struct commands xc_cmds[] = {
 	 N_("KICK <nick>, kicks the nick from the current channel (needs chanop)")},
 	{"KICKBAN", cmd_kickban, 1, 1,
 	 N_("KICKBAN <nick>, bans then kicks the nick from the current channel (needs chanop)")},
+	{"KILLALL", cmd_killall, 0, 0, "KILLALL, immediately exit"},
 	{"LAGCHECK", cmd_lagcheck, 0, 0,
 	 N_("LAGCHECK, forces a new lag check")},
 	{"LASTLOG", cmd_lastlog, 0, 0,
@@ -3056,12 +3108,17 @@ user_command (session * sess, char *tbuf, char *cmd, char *word[],
 static void
 handle_say (session *sess, char *text, int check_spch)
 {
-	char tbuf[4096];
-	char newcmd[2048];
 	struct DCC *dcc;
-	char pdibuf[2048];
 	char *word[PDIWORDS];
 	char *word_eol[PDIWORDS];
+	char pdibuf_static[1024];
+	char newcmd_static[1024];
+	char tbuf_static[4096];
+	char *pdibuf = pdibuf_static;
+	char *newcmd = newcmd_static;
+	char *tbuf = tbuf_static;
+	int len;
+	int newcmdlen = sizeof newcmd_static;
 
 	if (strcmp (sess->channel, "(lastlog)") == 0)
 	{
@@ -3069,6 +3126,16 @@ handle_say (session *sess, char *text, int check_spch)
 		return;
 	}
 
+	len = strlen (text);
+	if (len >= sizeof pdibuf_static)
+		pdibuf = malloc (len + 1);
+
+	if (len + NICKLEN >= newcmdlen)
+		newcmd = malloc (newcmdlen = len + NICKLEN + 1);
+
+	if (len * 2 >= sizeof tbuf_static)
+		tbuf = malloc (len * 2 + 1);
+
 	if (check_spch && prefs.perc_color)
 		check_special_chars (text, prefs.perc_ascii);
 
@@ -3077,18 +3144,22 @@ handle_say (session *sess, char *text, int check_spch)
 
 	/* a command of "" can be hooked for non-commands */
 	if (plugin_emit_command (sess, "", word, word_eol))
-		return;
+		goto xit;
+
+	/* incase a plugin did /close */
+	if (!is_session (sess))
+		goto xit;
 
 	if (!sess->channel[0] || sess->type == SESS_SERVER || sess->type == SESS_NOTICES || sess->type == SESS_SNOTICES)
 	{
 		notj_msg (sess);
-		return;
+		goto xit;
 	}
 
 	if (prefs.nickcompletion)
 		perform_nick_completion (sess, text, newcmd);
 	else
-		safe_strcpy (newcmd, text, sizeof (newcmd));
+		safe_strcpy (newcmd, text, newcmdlen);
 
 	text = newcmd;
 
@@ -3098,10 +3169,10 @@ handle_say (session *sess, char *text, int check_spch)
 		dcc = dcc_write_chat (sess->channel, text);
 		if (dcc)
 		{
-			inbound_chanmsg (sess->server, tbuf, sess->channel,
+			inbound_chanmsg (sess->server, sess->channel,
 								  sess->server->nick, text, TRUE);
 			set_topic (sess, net_ip (dcc->addr));
-			return;
+			goto xit;
 		}
 	}
 
@@ -3111,19 +3182,39 @@ handle_say (session *sess, char *text, int check_spch)
 		unsigned char t = 0;
 
 		/* maximum allowed message text */
-		/* PRIVMSG #channel :text\r\n */
-		/* 12345678        90    12 */
-		max = 456 - strlen (sess->channel);
+		/* :nickname!username@host.com PRIVMSG #channel :text\r\n */
+		max = 512;
+		max -= 16;	/* :, !, @, " PRIVMSG ", " ", :, \r, \n */
+		max -= strlen (sess->server->nick);
+		max -= strlen (sess->channel);
+		if (sess->me && sess->me->hostname)
+			max -= strlen (sess->me->hostname);
+		else
+		{
+			max -= 9;	/* username */
+			max -= 65;	/* max possible hostname and '@' */
+		}
 
 		if (strlen (text) > max)
 		{
-			/* FIXME: utf8 */
+			int i = 0, size;
+
+			/* traverse the utf8 string and find the nearest cut point that
+				doesn't split 1 char in half */
+			while (1)
+			{
+				size = g_utf8_skip[((unsigned char *)text)[i]];
+				if ((i + size) >= max)
+					break;
+				i += size;
+			}
+			max = i;
 			t = text[max];
 			text[max] = 0;			  /* insert a NULL terminator to shorten it */
 		}
 
-		inbound_chanmsg (sess->server, tbuf, sess->channel,
-						 sess->server->nick, text, TRUE);
+		inbound_chanmsg (sess->server, sess->channel, sess->server->nick,
+							  text, TRUE);
 		sess->server->p_message (sess->server, sess->channel, text);
 
 		if (t)
@@ -3136,6 +3227,16 @@ handle_say (session *sess, char *text, int check_spch)
 	{
 		notc_msg (sess);
 	}
+
+xit:
+	if (pdibuf != pdibuf_static)
+		free (pdibuf);
+
+	if (newcmd != newcmd_static)
+		free (newcmd);
+
+	if (tbuf != tbuf_static)
+		free (tbuf);
 }
 
 /* handle a command, without the '/' prefix */
@@ -3146,12 +3247,16 @@ handle_command (session *sess, char *cmd, int check_spch)
 	struct popup *pop;
 	int user_cmd = FALSE;
 	GSList *list;
-	char pdibuf[2048];
-	char tbuf[4096];
 	char *word[PDIWORDS];
 	char *word_eol[PDIWORDS];
 	static int command_level = 0;
 	struct commands *int_cmd;
+	char pdibuf_static[1024];
+	char tbuf_static[4096];
+	char *pdibuf;
+	char *tbuf;
+	int len;
+	int ret = TRUE;
 
 	if (command_level > 99)
 	{
@@ -3161,6 +3266,17 @@ handle_command (session *sess, char *cmd, int check_spch)
 	command_level++;
 	/* anything below MUST DEC command_level before returning */
 
+	len = strlen (cmd);
+	if (len >= sizeof (pdibuf_static))
+		pdibuf = malloc (len + 1);
+	else
+		pdibuf = pdibuf_static;
+
+	if ((len * 2) >= sizeof (tbuf_static))
+		tbuf = malloc ((len * 2) + 1);
+	else
+		tbuf = tbuf_static;
+
 	/* split the text into words and word_eol */
 	process_data_init (pdibuf, cmd, word, word_eol, TRUE);
 
@@ -3170,6 +3286,10 @@ handle_command (session *sess, char *cmd, int check_spch)
 	if (plugin_emit_command (sess, word[1], word, word_eol))
 		goto xit;
 
+	/* incase a plugin did /close */
+	if (!is_session (sess))
+		goto xit;
+
 	/* first see if it's a userCommand */
 	list = command_list;
 	while (list)
@@ -3205,8 +3325,8 @@ handle_command (session *sess, char *cmd, int check_spch)
 				help (sess, tbuf, int_cmd->name, TRUE);
 				break;
 			case 2:
-				command_level--;
-				return FALSE;
+				ret = FALSE;
+				goto xit;
 			}
 		}
 	} else
@@ -3220,7 +3340,14 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 xit:
 	command_level--;
-	return TRUE;
+
+	if (pdibuf != pdibuf_static)
+		free (pdibuf);
+
+	if (tbuf != tbuf_static)
+		free (tbuf);
+
+	return ret;
 }
 
 /* handle one line entered into the input box */
