@@ -164,6 +164,15 @@ irc_user_list (server *serv, char *channel)
 	tcp_sendf (serv, "WHO %s\r\n", channel);
 }
 
+static void
+irc_away_status (server *serv, char *channel)
+{
+	if (serv->have_whox)
+		tcp_sendf (serv, "WHO %s %%cnf\r\n", channel);
+	else
+		tcp_sendf (serv, "WHO %s\r\n", channel);
+}
+
 /*static void
 irc_get_ip (server *serv, char *nick)
 {
@@ -282,7 +291,7 @@ channel_date (session *sess, char *chan, char *timestr)
 }
 
 static void
-process_numeric (session * sess, char *outbuf, int n,
+process_numeric (session * sess, int n,
 					  char *word[], char *word_eol[], char *text)
 {
 	session *realsess;
@@ -346,6 +355,7 @@ process_numeric (session * sess, char *outbuf, int n,
 		/* FALL THROUGH */
 
 	case 314:
+		inbound_user_info_start (sess, word[4]);
 		EMIT_SIGNAL (XP_TE_WHOIS1, serv->server_session, word[4], word[5],
 						 word[6], word_eol[8] + 1, 0);
 		break;
@@ -355,7 +365,10 @@ process_numeric (session * sess, char *outbuf, int n,
 			long n = atol (word[6]);
 			long idle = atol (word[5]);
 			char *tim;
-			sprintf (outbuf, "%02ld:%02ld:%02ld", idle / 3600, (idle / 60) % 60,
+			char outbuf[64];
+
+			snprintf (outbuf, sizeof (outbuf),
+						"%02ld:%02ld:%02ld", idle / 3600, (idle / 60) % 60,
 						idle % 60);
 			if (n == 0)
 				EMIT_SIGNAL (XP_TE_WHOIS4, serv->server_session, word[4],
@@ -382,6 +395,12 @@ process_numeric (session * sess, char *outbuf, int n,
 						 word_eol[5] + 1, NULL, NULL, 0);
 		break;
 
+	case 307:	/* dalnet version */
+	case 320:	/* :is an identified user */
+		EMIT_SIGNAL (XP_TE_WHOIS_ID, serv->server_session, word[4],
+						 word_eol[5] + 1, NULL, NULL, 0);
+		break;
+
 	case 321:
 		if (!fe_is_chanwindow (sess->server))
 			EMIT_SIGNAL (XP_TE_CHANLISTHEAD, sess, NULL, NULL, NULL, NULL, 0);
@@ -400,8 +419,9 @@ process_numeric (session * sess, char *outbuf, int n,
 
 	case 323:
 		if (!fe_is_chanwindow (sess->server))
-			goto def;
-		fe_chan_list_end (sess->server);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, sess, text, word[1], NULL, NULL, 0);
+		else
+			fe_chan_list_end (sess->server);
 		break;
 
 	case 324:
@@ -435,6 +455,11 @@ process_numeric (session * sess, char *outbuf, int n,
 		}
 		break;
 
+	case 330:
+		EMIT_SIGNAL (XP_TE_WHOIS_AUTH, serv->server_session, word[4],
+			word_eol[6] + 1, word[5], NULL, 0);
+                break;
+
 	case 332:
 		inbound_topic (serv, word[4],
 						(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5]);
@@ -444,6 +469,11 @@ process_numeric (session * sess, char *outbuf, int n,
 		inbound_topictime (serv, word[4], word[5], atol (word[6]));
 		break;
 
+	case 338:  /* Undernet Real user@host, Real IP */
+		EMIT_SIGNAL (XP_TE_WHOIS_REALHOST, sess, word[4], word[5], word[6], 
+			(word_eol[7][0]==':') ? word_eol[7]+1 : word_eol[7], 0);
+		break;
+
 	case 341:						  /* INVITE ACK */
 		EMIT_SIGNAL (XP_TE_UINVITE, sess, word[4], word[5], serv->servername,
 						 NULL, 0);
@@ -452,8 +482,17 @@ process_numeric (session * sess, char *outbuf, int n,
 	case 352:						  /* WHO */
 		if (!serv->skip_next_who)
 		{
-			if (!inbound_user_info (sess, outbuf, word[4], word[5], word[6],
-											word[7], word[8], word_eol[11]))
+			unsigned int away = 0;
+			session *who_sess = find_channel (serv, word[4]);
+
+			if (strchr (word[9], 'G'))
+				away = 1;
+
+			inbound_user_info (sess, word[4], word[5], word[6], word[7],
+									 word[8], word_eol[11], away);
+
+			/* try to show only user initiated whos */
+			if (!who_sess || !who_sess->doing_who)
 				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1],
 								 NULL, NULL, 0);
 		} else
@@ -463,6 +502,24 @@ process_numeric (session * sess, char *outbuf, int n,
 		}
 		break;
 
+	case 354:	/* undernet WHOX: used as a reply for irc_away_status */
+		{
+			unsigned int away = 0;
+			session *who_sess = find_channel (serv, word[4]);
+
+			if (strchr (word[6], 'G'))
+				away = 1;
+
+			/* :SanJose.CA.us.undernet.org 354 z1 #zed1 z1 H@ */
+			inbound_user_info (sess, word[4], 0, 0, 0, word[5], 0, away);
+
+			/* try to show only user initiated whos */
+			if (!who_sess || !who_sess->doing_who)
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1],
+								 NULL, NULL, 0);
+		}
+		break;
+
 	case 315:						  /* END OF WHO */
 		if (serv->skip_next_who)
 		{
@@ -474,13 +531,15 @@ process_numeric (session * sess, char *outbuf, int n,
 			if (who_sess)
 			{
 				if (!who_sess->doing_who)
-					goto def;
+					EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text,
+									 word[1], NULL, NULL, 0);
 				who_sess->doing_who = FALSE;
 			} else
 			{
-				if (!serv->doing_who)
-					goto def;
-				serv->doing_who = FALSE;
+				if (!serv->doing_dns)
+					EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text,
+									 word[1], NULL, NULL, 0);
+				serv->doing_dns = FALSE;
 			}
 		}
 		break;
@@ -580,8 +639,7 @@ process_numeric (session * sess, char *outbuf, int n,
 /* handle named messages that starts with a ':' */
 
 static void
-process_named_msg (session *sess, char *type, char *outbuf,
-					    char *word[], char *word_eol[])
+process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 {
 	server *serv = sess->server;
 	char ip[128], nick[NICKLEN];
@@ -636,7 +694,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 	if (!strcmp ("NOTICE", type))
 	{
 		if (!ignore_check (word[1], IG_NOTI))
-			inbound_notice (serv, outbuf, word[3], nick,
+			inbound_notice (serv, word[3], nick,
 					(word_eol[4][0] == ':') ? word_eol[4] + 1 : word_eol[4], ip);
 		return;
 	}
@@ -676,19 +734,19 @@ process_named_msg (session *sess, char *type, char *outbuf,
 				if (strncasecmp (text, "DCC ", 4) == 0)
 					/* redo this with handle_quotes TRUE */
 					process_data_init (word[1], word_eol[1], word, word_eol, TRUE);
-				ctcp_handle (sess, outbuf, to, nick, text, word, word_eol);
+				ctcp_handle (sess, to, nick, text, word, word_eol);
 			} else
 			{
 				if (is_channel (serv, to))
 				{
 					if (ignore_check (word[1], IG_CHAN))
 						return;
-					inbound_chanmsg (serv, outbuf, to, nick, text, FALSE);
+					inbound_chanmsg (serv, to, nick, text, FALSE);
 				} else
 				{
 					if (ignore_check (word[1], IG_PRIV))
 						return;
-					inbound_privmsg (serv, outbuf, nick, ip, text);
+					inbound_privmsg (serv, nick, ip, text);
 				}
 			}
 		}
@@ -696,7 +754,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 	}
 	if (!strcmp ("PONG", type))
 	{
-		inbound_ping_reply (serv->server_session, outbuf,
+		inbound_ping_reply (serv->server_session,
 							 (word[4][0] == ':') ? word[4] + 1 : word[4], word[3]);
 		return;
 	}
@@ -742,8 +800,7 @@ process_named_msg (session *sess, char *type, char *outbuf,
 	}
 
 	/* unknown message */
-	sprintf (outbuf, "GARBAGE: %s\n", word_eol[1]);
-	PrintText (sess, outbuf);
+	PrintTextf (sess, "GARBAGE: %s\n", word_eol[1]);
 }
 
 /* handle named messages that DON'T start with a ':' */
@@ -783,7 +840,6 @@ irc_inline (server *serv, char *buf, int len)
 	char *type, *text;
 	char *word[PDIWORDS];
 	char *word_eol[PDIWORDS];
-	char outbuf[4096];
 	char pdibuf_static[522]; /* 1 line can potentially be 512*6 in utf8 */
 	char *pdibuf = pdibuf_static;
 
@@ -837,10 +893,10 @@ irc_inline (server *serv, char *buf, int len)
 		if (*text == ':')
 			text++;
 
-		process_numeric (sess, outbuf, atoi (word[2]), word, word_eol, text);
+		process_numeric (sess, atoi (word[2]), word, word_eol, text);
 	} else
 	{
-		process_named_msg (sess, type, outbuf, word, word_eol);
+		process_named_msg (sess, type, word, word_eol);
 	}
 
 xit:
@@ -865,6 +921,7 @@ proto_fill_her_up (server *serv)
 	serv->p_chan_mode = irc_chan_mode;
 	serv->p_nick_mode = irc_nick_mode;
 	serv->p_user_list = irc_user_list;
+	serv->p_away_status = irc_away_status;
 	/*serv->p_get_ip = irc_get_ip;*/
 	serv->p_whois = irc_user_whois;
 	serv->p_get_ip = irc_user_list;
