@@ -61,6 +61,19 @@ static const struct defaultserver def[] =
 	{0,			"baltimore.md.us.afternet.org"},
 	{0,			"boston.afternet.org"},
 
+	{"Aitvaras",	0},
+#ifdef USE_IPV6
+	{0,			"irc6.ktu.lt"},
+#endif
+	{0,			"irc.delfi.lt"},
+	{0,			"irc.ktu.lt"},
+	{0,			"irc.le.lt"},
+	{0,			"irc.takas.lt"},
+	{0,			"irc.omnitel.net"},
+	{0,			"irc.5ci.net"},
+	{0,			"irc.interneka.lt"},
+	{0,			"irc.elekta.lt"},
+
 	{"AmigaNet",	0},
 	{0,			"linux.us.amiganet.org"},
 	{0,			"whiterose.us.amiganet.org"},
@@ -407,6 +420,11 @@ static const struct defaultserver def[] =
 	{0,			"moo.slashnet.org"},
 	{0,			"radon.slashnet.org"},
 
+	{"SorceryNet",	0},
+	{0,			"irc.sorcery.net/9000"},
+	{0,			"irc.us.sorcery.net/9000"},
+	{0,			"irc.eu.sorcery.net/9000"},
+
 	{"Spidernet",	0},
 	{0,			"us.spidernet.org"},
 	{0,			"eu.spidernet.org"},
@@ -424,6 +442,7 @@ static const struct defaultserver def[] =
 	{0,			"irc.subcult.ch"},
 	{0,			"irc.phuncrew.ch"},
 	{0,			"irc.mgz.ch"},
+	{0,			"irc.nazgul.ch/6666"},
 
 	{"TNI3",			0},
 	{0,			"irc.tni3.com"},
