@@ -141,6 +141,48 @@ static char *strhebpatch(char *dest, const char *src)
 
 #endif /* !USE_HEBREW */
 
+/* Make ip:port urls clikable (127.0.0.1:80 etc)
+*  patch by Alex <alex@cosinus.org> & dobler <dobler@barrysworld.com>
+*/
+
+static int
+q3link (char *word)
+{
+	char *s;
+	int i;
+	int d = 0;
+
+	if ((s = strchr (word,':')) != NULL)
+	{
+		for (i = 0; i < (int) ((unsigned long)s - (unsigned long)word); i++)
+		{
+			if (word[i] == '.')
+				d++;
+			else if (!isdigit (word[i]))
+			{
+				d = 0;
+				break;
+			}
+		}
+
+		if (d == 3)
+		{
+			s++;
+			d = 0;
+
+			while (*s != 0)
+			{
+				if (!isdigit (*s++))
+					return (0);
+				d++;
+			}
+			if (d > 0)
+				return (1);
+		}
+	}
+	return (0);
+}
+
 /* check if a word is clickable */
 
 int
@@ -148,9 +190,11 @@ text_word_check (char *word)
 {
 	session *sess = current_sess;
 	char *at, *dot;
-	int i, dots;
 	int len = strlen (word);
 
+	if(q3link(word))
+		return WORD_URL;
+
 	if ((word[0] == '@' || word[0] == '+') && word[1] == '#')
 		return WORD_CHANNEL;
 
@@ -184,7 +228,7 @@ text_word_check (char *word)
 	if (!strncasecmp (word, "gopher://", 9) && word[9] != 0)
 		return WORD_URL;
 
-	if (find_name (sess, word))
+	if (( (word[0]=='@' || word[0]=='+') && find_name (sess, word+1)) || find_name (sess, word))
 		return WORD_NICK;
 
 	at = strchr (word, '@');	  /* check for email addy */
@@ -200,21 +244,6 @@ text_word_check (char *word)
 		}
 	}
 
-	/* check if it's an IP number */
-	dots = 0;
-	for (i = 0; i < len; i++)
-	{
-		if (word[i] == '.')
-			dots++;
-		else if (!isdigit (word[i]))
-		{
-			dots = 0;
-			break;
-		}
-	}
-	if (dots == 3)
-		return WORD_HOST;
-
 	if (!strncasecmp (word + len - 5, ".html", 5))
 		return WORD_HOST;
 
@@ -279,7 +308,7 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 			if (mbl == 1)
 			{
 #ifndef WIN32
-				*tmp = tolower (*tmp);
+				*tmp = rfc_tolower (*tmp);
 				if (*tmp == '/')
 #else
 				/* win32 can't handle filenames with the '|' character */
@@ -443,7 +472,7 @@ text_validate (char **text, int *len)
 	{
 		if (error)
 		{
-			*text = g_strdup_printf ("\0034ERROR\017\t%s\n", error->message);
+			*text = g_strdup_printf ("\0034ICONV\017 %s\n", error->message);
 			*len = strlen (*text);
 			g_error_free (error);
 		} else
@@ -831,6 +860,24 @@ static char *pevt_whois6_help[] = {
 	N_("Nickname"),
 };
 
+static char *pevt_whoisid_help[] = {
+	N_("Nickname"),
+	N_("Message"),
+};
+
+static char *pevt_whoisauth_help[] = {
+	N_("Nickname"),
+	N_("Message"),
+	N_("Account"),
+};
+
+static char *pevt_whoisrealhost_help[] = {
+	N_("Nickname"),
+	N_("Real user@host"),
+	N_("Real IP"),
+	N_("Message"),
+};
+
 static char *pevt_generic_channel_help[] = {
 	N_("Channel Name"),
 };
@@ -1509,6 +1556,33 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
+static void
+play_wave (const char *file)
+{
+	char buf[512];
+	char wavfile[512];
+
+	memset (buf, 0, sizeof (buf));
+	memset (wavfile, 0, sizeof (wavfile));
+	if (file[0] != '/')
+	{
+		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
+	} else
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	}
+	if (access (wavfile, R_OK) == 0)
+	{
+		snprintf (buf, sizeof (buf), "%s %s", prefs.soundcmd, wavfile);
+		buf[sizeof (buf) - 1] = '\0';
+		xchat_exec (buf);
+	} else
+	{
+		snprintf (buf, sizeof (buf), "Cannot read %s", wavfile);
+		fe_message (buf, FALSE);
+	}
+}
+
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1529,7 +1603,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 		return;
 
 	if (te[index].sound)
-		fe_play_wave (te[index].sound);
+		play_wave (te[index].sound);
 
 	display_event (pntevts[index], sess, te[index].num_args, word);
 }
