@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-
+#define _FILE_OFFSET_BITS 64
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -26,6 +26,8 @@
 #include <sys/timeb.h>
 #include <process.h>
 #else
+#include <sys/types.h>
+#include <pwd.h>
 #include <sys/time.h>
 #include <sys/utsname.h>
 #endif
@@ -334,12 +336,32 @@ char *
 expand_homedir (char *file)
 {
 #ifndef WIN32
-	char *ret;
+	char *ret, *user;
+	struct passwd *pw;
 
 	if (*file == '~')
 	{
-		ret = malloc (strlen (file) + strlen (g_get_home_dir ()) + 1);
-		sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
+		if (file[1] != '\0' && file[1] != '/')
+		{
+			user = strdup(file);
+			if (strchr(user,'/') != NULL)
+				*(strchr(user,'/')) = '\0';
+			if ((pw = getpwnam(user + 1)) == NULL)
+			{
+				free(user);
+				return strdup(file);
+			}
+			free(user);
+			user = strchr(file, '/') != NULL ? strchr(file,'/') : file;
+			ret = malloc(strlen(user) + strlen(pw->pw_dir) + 1);
+			strcpy(ret, pw->pw_dir);
+			strcat(ret, user);
+		}
+		else
+		{
+			ret = malloc (strlen (file) + strlen (g_get_home_dir ()) + 1);
+			sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
+		}
 		return ret;
 	}
 #endif
@@ -1236,23 +1258,39 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 	char *output_name, int dccpermissions)
 {
 	/* Move a complete file to move_file_path */
+	/* mgl: this used to take just a filename and move it between dirs */
+	/* now it takes the full path of the target download and moves it */
 
 	char dl_src[4096];
 	char dl_dest[4096];
 	char dl_tmp[4096];
+	char *output_fname;
 	int return_tmp, return_tmp2;
-	struct stat buf;
 
 	/* if dcc_dir and dcc_completed_dir are the same then we are done */
 	if (0 == strcmp (dcc_dir, dcc_completed_dir) ||
 		 0 == dcc_completed_dir[0])
 		return;			/* Already in "completed dir" */
 
-	snprintf (dl_src, sizeof (dl_src), "%s/%s", dcc_dir, output_name);
-	snprintf (dl_dest, sizeof (dl_dest), "%s/%s", dcc_completed_dir,
-			   output_name);
-
+	/* mgl: since output_name is a full path, we just copy it */
+	strncpy (dl_src, output_name, sizeof(dl_src));
 	dl_src[sizeof(dl_src)-1] = '\0';
+
+	/* mgl: output_name being a full path, we need to extract the filename */
+	/* off the end of it before continuing */
+
+	/* no path sep or no file after pathsep?  very suspicious, bail now! */
+	if ((NULL == (output_fname = strrchr(output_name, '/')))
+			|| !*(output_fname + 1))
+		return;
+	/* get the next char after the pathsep */
+	++output_fname;
+	/* throw the filename onto the directory name of the completed directory */
+	/* FIXME: dcc_completed_dir is UTF8, not fs encoding! */
+	snprintf (dl_dest, sizeof (dl_dest), "%s/%s", dcc_completed_dir,
+			   output_fname);
+	/* the rest should continue as before, but use output_fname */
+
 	dl_dest[sizeof(dl_dest)-1] = '\0';
 
 	/*
@@ -1266,10 +1304,11 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 	 *		--RAM, 03/11/2001
 	 */
 
-	if (-1 != stat (dl_dest, &buf))
+	if (access (dl_dest, F_OK) == 0)
 	{
 		int destlen = strlen (dl_dest);
 		int i;
+		struct stat buf;
 
 		/*
 		 * There must be enough room for us to append the ".xx" extensions.
@@ -1279,7 +1318,7 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 		if (destlen >= sizeof (dl_dest) - 4)
 		{
 			fprintf(stderr, "Found '%s' in completed dir, and path already too long",
-				output_name);
+				output_fname);
 			return;
 		}
 
@@ -1300,7 +1339,7 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 		{
 			fprintf (stderr, "Found '%s' in completed dir, "
 				"and was unable to find another unique name",
-				output_name);
+				output_fname);
 			return;
 		}
 
@@ -1323,7 +1362,7 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 		if ((tmp_src = open (dl_src, O_RDONLY | OFLAGS)) == -1)
 		{
 			fprintf (stderr, "Unable to open() file '%s' (%s) !", dl_src,
-					  g_strerror (errno));
+					  strerror (errno));
 			return;
 		}
 
@@ -1377,21 +1416,3 @@ download_move_to_completed_dir (char *dcc_dir, char *dcc_completed_dir,
 			unlink (dl_src);
 	}
 }
-
-void
-play_wave (const char *file)
-{
-	char buf[512];
-
-	snprintf (buf, sizeof (buf), "%s/%s", prefs.sounddir, file);
-	if (access (buf, R_OK) == 0)
-	{
-		snprintf (buf, sizeof (buf), "%s %s/%s", prefs.soundcmd, prefs.sounddir, file);
-		buf[sizeof (buf) - 1] = '\0';
-		xchat_exec (buf);
-	} else
-	{
-		snprintf (buf, sizeof (buf), "Cannot read %s/%s", prefs.sounddir, file);
-		fe_message (buf, FALSE);
-	}
-}
