@@ -70,6 +70,7 @@ static GSList *away_list = 0;
 static int in_xchat_exit = FALSE;
 int xchat_is_quitting = FALSE;
 int auto_connect = TRUE;
+int skip_plugins = FALSE;
 char *connect_url = NULL;
 
 struct session *current_tab;
@@ -185,6 +186,67 @@ lag_check (void)
 	}
 }
 
+static int
+away_check (void)
+{
+	session *sess;
+	GSList *list;
+	int full, sent, loop = 0;
+
+	if (prefs.away_size_max < 1)
+		return 1;
+
+doover:
+	/* request an update of AWAY status of 1 channel every 30 seconds */
+	full = TRUE;
+	sent = 0;	/* number of WHOs (users) requested */
+	list = sess_list;
+	while (list)
+	{
+		sess = list->data;
+
+		if (sess->server->connected &&
+			 sess->type == SESS_CHANNEL &&
+			 sess->channel[0] &&
+			 sess->total <= prefs.away_size_max)
+		{
+			if (!sess->done_away_check)
+			{
+				full = FALSE;
+
+				/* if we're under 31 WHOs, send another channels worth */
+				if (sent < 31 && !sess->doing_who)
+				{
+					sess->done_away_check = TRUE;
+					sess->doing_who = TRUE;
+					/* this'll send a WHO #channel */
+					sess->server->p_away_status (sess->server, sess->channel);
+					sent += sess->total;
+				}
+			}
+		}
+
+		list = list->next;
+	}
+
+	/* done them all, reset done_away_check to FALSE and start over */
+	if (full)
+	{
+		list = sess_list;
+		while (list)
+		{
+			sess = list->data;
+			sess->done_away_check = FALSE;
+			list = list->next;
+		}
+		loop++;
+		if (loop < 2)
+			goto doover;
+	}
+
+	return 1;
+}
+
 static int
 xchat_misc_checks (void)		/* this gets called every 1/2 second */
 {
@@ -220,7 +282,8 @@ irc_init (session *sess)
 	done_init = TRUE;
 
 #ifdef USE_PLUGIN
-	plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
+	if (!skip_plugins)
+		plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
 #endif
 	plugin_add (sess, NULL, NULL, timer_plugin_init, NULL, NULL, FALSE);
 
@@ -228,6 +291,7 @@ irc_init (session *sess)
 		notify_tag = fe_timeout_add (prefs.notify_timeout * 1000,
 											  notify_checklist, 0);
 
+	fe_timeout_add (prefs.away_timeout * 1000, away_check, 0);
 	fe_timeout_add (500, xchat_misc_checks, 0);
 
 	if (connect_url != NULL)
@@ -611,13 +675,6 @@ find_away_message (struct server *serv, char *nick)
 	"NAME TIME\n"				"CMD nctcp %s TIME %t\n\n"\
 	"NAME PING\n"				"CMD nctcp %s PING %d\n\n"
 
-#define defaultconf_dlgbuttons \
-	"NAME Whois\n"				"CMD whois %s\n\n"\
-	"NAME Send\n"				"CMD dcc send %s\n\n"\
-	"NAME Chat\n"				"CMD dcc chat %s\n\n"\
-	"NAME Ping\n"				"CMD ping %s\n\n"\
-	"NAME Clear\n"				"CMD clear\n\n"
-
 #define defaultconf_replace \
 	"NAME teh\n"				"CMD the\n\n"
 /*	"NAME r\n"					"CMD are\n\n"\
@@ -626,6 +683,7 @@ find_away_message (struct server *serv, char *nick)
 #define defaultconf_commands \
 	"NAME ACTION\n"		"CMD me &2\n\n"\
 	"NAME AME\n"			"CMD allchan me &2\n\n"\
+	"NAME ANICK\n"			"CMD allserv nick &2\n\n"\
 	"NAME AMSG\n"			"CMD allchan say &2\n\n"\
 	"NAME BACK\n"			"CMD away\n\n"\
 	"NAME BANLIST\n"		"CMD quote MODE %c +b\n\n"\
@@ -897,8 +955,20 @@ xchat_init (void)
 				_("Dialog"));
 	list_loadconf ("buttons.conf", &button_list, buf);
 
+	snprintf (buf, sizeof (buf),
+		"NAME %s\n"				"CMD whois %%s\n\n"
+		"NAME %s\n"				"CMD dcc send %%s\n\n"
+		"NAME %s\n"				"CMD dcc chat %%s\n\n"
+		"NAME %s\n"				"CMD ping %%s\n\n"
+		"NAME %s\n"				"CMD clear\n\n",
+				_("WhoIs"),
+				_("Send"),
+				_("Chat"),
+				_("Ping"),
+				_("Clear"));
+	list_loadconf ("dlgbuttons.conf", &dlgbutton_list, buf);
+
 	list_loadconf ("ctcpreply.conf", &ctcp_list, defaultconf_ctcp);
-	list_loadconf ("dlgbuttons.conf", &dlgbutton_list, defaultconf_dlgbuttons);
 	list_loadconf ("commands.conf", &command_list, defaultconf_commands);
 	list_loadconf ("replace.conf", &replace_list, defaultconf_replace);
 	list_loadconf ("urlhandlers.conf", &urlhandler_list,
