@@ -19,10 +19,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
@@ -412,10 +408,14 @@ chanlist_wild (GtkWidget * wid, struct server *serv)
 #else
 	/* recompile the regular expression. */
 	if (serv->gui->have_regex)
+	{
+		serv->gui->have_regex = 0;
 		regfree (&serv->gui->chanlist_match_regex);
-	regcomp (&serv->gui->chanlist_match_regex, pattern,
-				REG_ICASE | REG_EXTENDED | REG_NOSUB);
-	serv->gui->have_regex = 1;
+	}
+
+	if (regcomp (&serv->gui->chanlist_match_regex, pattern,
+					 REG_ICASE | REG_EXTENDED | REG_NOSUB) == 0)
+		serv->gui->have_regex = 1;
 #endif
 }
 
@@ -590,10 +590,14 @@ chanlist_closegui (GtkWidget *wid, server *serv)
 void
 chanlist_opengui (struct server *serv)
 {
-	gchar *titles[] = { _("Channel"), _("Users"), _("Topic") };
+	gchar *titles[3];
 	GtkWidget *frame, *vbox, *hbox, *table, *wid;
 	char tbuf[256];
 
+	titles[0] = _("Channel");
+	titles[1] = _("Users");
+	titles[2] = _("Topic");
+
 	if (serv->gui->chanlist_window)
 	{
 		mg_bring_tofront (serv->gui->chanlist_window);
