@@ -20,10 +20,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -101,11 +97,14 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, gpointer data)
 	Atom at = None;
 	XEvent *xevent = (XEvent *)xev;
 
-	if (at == None)
-		at = XInternAtom (GDK_DISPLAY (), "_XROOTPMAP_ID", True);
+	if (xevent->type == PropertyNotify)
+	{
+		if (at == None)
+			at = XInternAtom (xevent->xproperty.display, "_XROOTPMAP_ID", True);
 
-	if (xevent->type == PropertyNotify && at == xevent->xproperty.atom)
-		redraw_trans_xtexts ();
+		if (at == xevent->xproperty.atom)
+			redraw_trans_xtexts ();
+	}
 
 	return GDK_FILTER_CONTINUE;
 }
@@ -134,12 +133,20 @@ fe_args (int argc, char *argv[])
 		{
 			printf (PACKAGE" "VERSION"\n"
 					"Usage: %s [OPTIONS]... [URL]\n\n", argv[0]);
-			printf (_("Options:\n"
-					"  -d,  --cfgdir DIRECTORY   use a different config dir\n"
-					"  -a,  --no-auto            don't auto connect\n"
-					"  -v,  --version            show version information\n\n"
+			printf ("%s:\n"
+					"  -d,  --cfgdir %-11s %s\n"
+					"  -a,  --no-auto            %s\n"
+					"  -n,  --no-plugins         %s\n"
+					"  -v,  --version            %s\n\n"
 					"URL:\n"
-					"  irc://server:port/channel\n\n"));
+					"  irc://server:port/channel\n\n",
+						_("Options"),
+						_("DIRECTORY"),
+						_("use a different config dir"),
+						_("don't auto connect"),
+						_("don't auto load any plugins"),
+						_("show version information")
+					);
 			return 0;
 		}
 		if (!strcasecmp (argv[1], "-a") || !strcasecmp (argv[1], "--no-auto"))
@@ -147,6 +154,14 @@ fe_args (int argc, char *argv[])
 			auto_connect = 0;
 			offset++;
 		}
+		if (argc > 1 + offset)
+		{
+			if (!strcasecmp (argv[1+offset], "-n") || !strcasecmp (argv[1+offset], "--no-plugins"))
+			{
+				skip_plugins = 1;
+				offset++;
+			}
+		}
 	}
 
 	if (argc > 2 + offset)
@@ -163,8 +178,6 @@ fe_args (int argc, char *argv[])
 	if (argc > (offset + 1))
 		connect_url = strdup (argv[offset + 1]);
 
-	gtk_set_locale ();
-
 	gtk_init (&argc, &argv);
 
 #ifdef USE_XLIB
@@ -466,9 +479,7 @@ fe_chan_list_end (struct server *serv)
 void
 fe_notify_update (char *name)
 {
-	if (name)
-		update_all_of (name);
-	else
+	if (!name)
 		notify_gui_update ();
 }
 
@@ -657,7 +668,15 @@ fe_set_throttle (server *serv)
 }
 
 void
-fe_play_wave (const char *file)
+fe_ctrl_gui (session *sess, int action)
 {
-    play_wave (file);
+	switch (action)
+	{
+	case 0:
+		gtk_widget_hide (sess->gui->window); break;
+	case 1:
+		gtk_widget_show (sess->gui->window); break;
+	case 2:
+		mg_bring_tofront (sess->res->tab); break;
+	}
 }
