@@ -22,10 +22,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <fcntl.h>
 #include <ctype.h>
 
@@ -429,9 +425,14 @@ key_dialog_close ()
 static void
 key_dialog_add_new (GtkWidget * button, GtkCList * list)
 {
-	gchar *strs[] = { "", _("<none>"), _("<none>"), _("<none>"), _("<none>") };
+	gchar *strs[] = { "", NULL, NULL, NULL, NULL };
 	struct key_binding *kb;
 
+	strs[1] = _("<none>");
+	strs[2] = _("<none>");
+	strs[3] = _("<none>");
+	strs[4] = _("<none>");
+
 	kb = malloc (sizeof (struct key_binding));
 
 	kb->keyval = 0;
@@ -672,10 +673,14 @@ key_dialog_show ()
 {
 	GtkWidget *vbox, *hbox, *list, *vbox2, *wid, *wid2, *wid3, *hbox2;
 	struct key_binding *kb;
-	gchar *titles[] = { _("Mod"), _("Key"), _("Action"), "1", "2" };
+	gchar *titles[] = { NULL, NULL, NULL, "1", "2" };
 	char temp[32];
 	int i;
 
+	titles[0] = _("Mod");
+	titles[1] = _("Key");
+	titles[2] = _("Action");
+
 	if (key_dialog)
 	{
 		mg_bring_tofront (key_dialog);
@@ -783,13 +788,13 @@ key_dialog_show ()
 	gtk_box_pack_end (GTK_BOX (vbox), hbox2, 0, 0, 1);
 
 	wid = gtk_xtext_new (colors, 0);
-	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
 	gtk_xtext_set_tint (GTK_XTEXT (wid), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (GTK_XTEXT (wid),
 									  channelwin_pix,
 									  prefs.transparent, prefs.tint);
 	gtk_widget_set_usize (wid, 0, 75);
 	gtk_box_pack_start (GTK_BOX (hbox2), wid, 1, 1, 1);
+	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
 	gtk_widget_show (wid);
 
 	wid2 = gtk_vscrollbar_new (GTK_XTEXT (wid)->adj);
@@ -1455,11 +1460,11 @@ replace_handle (GtkWidget *t)
 }
 
 static int
-nick_comp_get_nick (char *tx, char *n)
+nick_comp_get_nick (char *tx, char *n, int n_size)
 {
 	int c, len = strlen (tx);
 
-	for (c = 0; c < len; c++)
+	for (c = 0; c < len && c < n_size; c++)
 	{
       if (tx[c] == ':' || tx[c] == ',' || tx[c] == prefs.nick_suffix[0])
 		{
@@ -1483,7 +1488,7 @@ nick_comp_chng (session *sess, GtkWidget * t, int updown)
 	GSList *head, *list;
 
 	text = gtk_entry_get_text (GTK_ENTRY (t));
-	if (nick_comp_get_nick ((char *)text, nick) == -1)
+	if (nick_comp_get_nick ((char *)text, nick, sizeof (nick)) == -1)
 		return;
 
 	len = strlen (nick);
@@ -1912,6 +1917,11 @@ tab_nick_comp (session *sess, GtkWidget *t, int shift)
 		nick_buf[0] = 0;
 		while (match_list)
 		{
+			if (strlen (nick_buf) + strlen (match_list->data) >= sizeof (nick_buf))
+			{
+				PrintText (sess, nick_buf);
+				nick_buf[0] = 0;		
+			}
 			sprintf (nick_buf, "%s%s ", nick_buf, (char *)match_list->data);
 			match_list = match_list->next;
 		}
@@ -1952,6 +1962,8 @@ tab_nick_comp (session *sess, GtkWidget *t, int shift)
 	gtk_editable_set_position (GTK_EDITABLE (t), cursor_pos);
 
 compdone:
+	if (match_list)
+		g_slist_free (match_list);
 	if (b4)
 		free (b4);
 	if (c5)
