@@ -1,3 +1,6 @@
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreemodel.h>
+
 typedef void (*filereqcallback) (void *, void *, char *file);
 
 void gtkutil_file_req (char *title, void *callback, void *userdata,
@@ -18,4 +21,12 @@ int gtkutil_clist_selection (GtkWidget * clist);
 void add_tip (GtkWidget * wid, char *text);
 void show_and_unfocus (GtkWidget * wid);
 void gtkutil_set_icon (GtkWidget *win);
-GtkWidget *gtkutil_window_new (char *title, int width, int height, int mouse_pos);
+GtkWidget *gtkutil_window_new (char *title, char *role, int width, int height, int flags);
+void gtkutil_copy_to_clipboard (GtkWidget *widget, GdkAtom selection,
+                                const gchar *str);
+GtkWidget *gtkutil_treeview_new (GtkWidget *box, GtkTreeModel *model,
+                                 GtkTreeCellDataFunc mapper, ...);
+gboolean gtkutil_treemodel_string_to_iter (GtkTreeModel *model, gchar *pathstr, GtkTreeIter *iter_ret);
+gboolean gtkutil_treeview_get_selected_iter (GtkTreeView *view, GtkTreeIter *iter_ret);
+gboolean gtkutil_treeview_get_selected (GtkTreeView *view, GtkTreeIter *iter_ret, ...);
+
