@@ -252,22 +252,28 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 	{
 		if (mg_gui)
 		{
-			g_idle_add ((GSourceFunc)tab_group_resize, mg_gui->tabs_box);
+/*			g_idle_add ((GSourceFunc)tab_group_resize, mg_gui->tabs_box);*/
 			if (prefs.mainwindow_save)
 			{
 				sess = current_sess;
-				gtk_window_get_position (GTK_WINDOW (mg_gui->window),
-												 &prefs.mainwindow_left,
-												 &prefs.mainwindow_top);
-				gtk_window_get_size (GTK_WINDOW (mg_gui->window),
-											&prefs.mainwindow_width,
-											&prefs.mainwindow_height);
+				prefs.mainwindow_left = event->x;
+				prefs.mainwindow_top = event->y;
+				prefs.mainwindow_width = event->width;
+				prefs.mainwindow_height = event->height;
 			}
 		}
 	}
 
 	if (sess)
 	{
+		if (sess->type == SESS_DIALOG && prefs.mainwindow_save)
+		{
+			prefs.dialog_left = event->x;
+			prefs.dialog_top = event->y;
+			prefs.dialog_width = event->width;
+			prefs.dialog_height = event->height;
+		}
+
 		if (((GtkXText *) sess->gui->xtext)->transparent)
 			gtk_widget_queue_draw (sess->gui->xtext);
 	}
@@ -895,7 +901,7 @@ mg_color_insert (GtkWidget *item, gpointer userdata)
 {
 	char buf[32];
 
-	sprintf (buf, "%%C%d", GPOINTER_TO_INT (userdata));
+	sprintf (buf, "%%C%02d", GPOINTER_TO_INT (userdata));
 	key_action_insert (current_sess->gui->input_box, 0, buf, 0, 0);
 }
 
@@ -934,6 +940,16 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 {
 	GtkWidget *menu, *submenu, *item;
 
+	/* shift-click to close a tab */
+	if (event->state & GDK_SHIFT_MASK)
+	{
+		if (sess)
+			mg_close_sess (sess);
+		else
+			gtk_widget_destroy (wid);
+		return FALSE;
+	}
+
 	if (event->button != 3)
 		return FALSE;
 
@@ -959,7 +975,7 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 
 		menu_toggle_item (_("Beep on message"), submenu, mg_beepmsg_cb, sess,
 								sess->beep);
-		if (sess->type != SESS_DIALOG)
+		if (sess->type == SESS_CHANNEL)
 			menu_toggle_item (_("Show join/part messages"), submenu, mg_hidejp_cb,
 									sess, !sess->hide_join_part);
 		menu_toggle_item (_("Color paste"), submenu, mg_colorpaste_cb, sess,
@@ -972,7 +988,7 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	if (sess)
 		mg_create_color_menu (menu, sess);
 
-	item = gtk_menu_item_new_with_label (_("Move to tab"));
+	item = gtk_menu_item_new_with_label (_("Go to"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	submenu = gtk_menu_new ();
@@ -1114,11 +1130,15 @@ static void
 mg_topic_cb (GtkWidget *entry, gpointer userdata)
 {
 	session *sess = current_sess;
+	char *text;
 
 	if (sess->channel[0] && sess->server->connected)
-		sess->server->p_topic (sess->server, sess->channel,
-									  GTK_ENTRY (entry)->text);
-	else
+	{
+		text = GTK_ENTRY (entry)->text;
+		if (text[0] == 0)
+			text = " ";
+		sess->server->p_topic (sess->server, sess->channel, text);
+	} else
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
 	/* restore focus to the input widget, where the next input will most
 likely be */
@@ -1628,7 +1648,8 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 		menu_urlmenu (even, word);
 		break;
 	case WORD_NICK:
-		menu_nickmenu (sess, even, word, FALSE);
+		menu_nickmenu (sess, even, (word[0]=='@' || word[0]=='+') ?
+			word+1 : word, FALSE);
 		break;
 	case WORD_CHANNEL:
 		if (*word == '@' || *word == '+')
@@ -1695,9 +1716,8 @@ mg_create_textarea (session_gui *gui, GtkWidget *box)
 	gtk_xtext_set_error_function (xtext, gtkutil_simpledialog);
 	gtk_xtext_set_urlcheck_function (xtext, mg_word_check);
 	gtk_xtext_set_max_lines (xtext, prefs.max_lines);
-	mg_update_xtext (GTK_WIDGET (xtext));
-
 	gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (xtext));
+	mg_update_xtext (GTK_WIDGET (xtext));
 
 	g_signal_connect (G_OBJECT (xtext), "word_click",
 							G_CALLBACK (mg_word_clicked), NULL);
@@ -1998,7 +2018,7 @@ mg_create_tabs (session_gui *gui, GtkWidget *box)
 	if (prefs.tabs_position == 2 || prefs.tabs_position == 3)
 		vert = TRUE;
 
-	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert);
+	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert, FALSE);
 	gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
 						1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 }
@@ -2044,8 +2064,13 @@ mg_create_topwindow (session *sess)
 	GtkWidget *vbox;
 	GtkWidget *vvbox;
 
-	win = gtkutil_window_new ("X-Chat ["VERSION"]", NULL, prefs.mainwindow_width,
-									  prefs.mainwindow_height, 0);
+	if (sess->type == SESS_DIALOG)
+		win = gtkutil_window_new ("X-Chat ["VERSION"]", NULL,
+										  prefs.dialog_width, prefs.dialog_height, 0);
+	else
+		win = gtkutil_window_new ("X-Chat ["VERSION"]", NULL,
+										  prefs.mainwindow_width,
+										  prefs.mainwindow_height, 0);
 	sess->gui->window = win;
 	gtk_container_set_border_width (GTK_CONTAINER (win), 2);
 	g_signal_connect (G_OBJECT (win), "focus_in_event",
@@ -2112,10 +2137,9 @@ mg_create_irctab (session *sess, GtkWidget *book)
 	session_gui *gui = sess->gui;
 
 	vbox = gtk_vbox_new (FALSE, 3);
+	gtk_notebook_append_page (GTK_NOTEBOOK (book), vbox, NULL);
 
 	mg_create_center (sess, gui, vbox);
-
-	gtk_notebook_append_page (GTK_NOTEBOOK (book), vbox, NULL);
 }
 
 static void
@@ -2183,11 +2207,11 @@ mg_create_tabwindow (session *sess)
 	sess->gui->window = win;
 	gtk_window_move (GTK_WINDOW (win), prefs.mainwindow_left,
 						  prefs.mainwindow_top);
-	gtk_window_set_default_size (GTK_WINDOW (win), prefs.mainwindow_width,
-										  prefs.mainwindow_height);
+/*	gtk_window_set_default_size (GTK_WINDOW (win), prefs.mainwindow_width,
+										  prefs.mainwindow_height);*/
 	gtk_container_set_border_width (GTK_CONTAINER (win), 2);
 
-	g_signal_connect (G_OBJECT (win), "delete-event",
+	g_signal_connect (G_OBJECT (win), "delete_event",
 						   G_CALLBACK (mg_tabwindow_de_cb), 0);
 	g_signal_connect (G_OBJECT (win), "destroy",
 						   G_CALLBACK (mg_tabwindow_kill_cb), 0);
@@ -2487,8 +2511,8 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 	if (first_run || prefs.newtabstofront)
 		tab_focus (res->tab);
 
-	while (g_main_pending ())
-		g_main_iteration (TRUE);
+/*	while (g_main_pending ())
+		g_main_iteration (TRUE);*/
 	tab_group_resize (mg_gui->tabs_box);
 	/*g_idle_add ((GSourceFunc)tab_group_resize, mg_gui->tabs_box);*/
 }
