@@ -38,6 +38,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtkversion.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "../common/xchat.h"
@@ -401,6 +402,11 @@ menu_destroy (GtkObject *object, gpointer objtounref)
 static void
 menu_popup (GtkWidget *menu, GdkEventButton *event, gpointer objtounref)
 {
+#if (GTK_MAJOR_VERSION != 2) || (GTK_MINOR_VERSION != 0)
+	if (event && event->window)
+		gtk_menu_set_screen (GTK_MENU (menu), gdk_drawable_get_screen (event->window));
+#endif
+
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (menu_destroy), objtounref);
 	if (event == NULL)
@@ -1289,6 +1295,9 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	GtkWidget *menu_bar;
 	GtkWidget *usermenu = 0;
 	GtkWidget *submenu = 0;
+	int close_mask = GDK_CONTROL_MASK;
+	char *key_theme = NULL;
+	GtkSettings *settings;
 
 	if (bar)
 		menu_bar = gtk_menu_bar_new ();
@@ -1307,6 +1316,19 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	mymenu[31-menuoffset].state = prefs.autodccsend;
 	/*mymenu[60-menuoffset].state = prefs.autosave;*/
 
+	/* change Close binding to ctrl-shift-w when using emacs keys */
+	settings = gtk_widget_get_settings (menu_bar);
+	if (settings)
+	{
+		g_object_get (settings, "gtk-key-theme-name", &key_theme, NULL);
+		if (key_theme)
+		{
+			if (!strcasecmp (key_theme, "Emacs"))
+				close_mask = GDK_SHIFT_MASK | GDK_CONTROL_MASK;
+			g_free (key_theme);
+		}
+	}
+
 	if (!toplevel)
 		mymenu[14-menuoffset].text = _("Detach Tab");
 	else
@@ -1342,8 +1364,8 @@ normalitem:
 				gtk_widget_add_accelerator (item, "activate", accel_group,
 										mymenu[i].key,
 										mymenu[i].key == GDK_F1 ? 0 :
-										mymenu[i].key == GDK_w ? GDK_CONTROL_MASK 
-										| GDK_SHIFT_MASK : GDK_CONTROL_MASK,
+										mymenu[i].key == GDK_w ? close_mask :
+										GDK_CONTROL_MASK,
 										GTK_ACCEL_VISIBLE);
 			if (mymenu[i].callback)
 				g_signal_connect (G_OBJECT (item), "activate",
