@@ -47,6 +47,7 @@ GtkStyle *create_input_style (void);
 
 
 static int last_selected_page = 0;
+static gboolean color_change;
 static struct xchatprefs setup_prefs;
 
 enum
@@ -132,6 +133,7 @@ static const setting userlist_settings[] =
 	{ST_TOGGLE, N_("Userlist buttons enabled"), P_OFFINT(userlistbuttons), 0, 0, 0},
 	{ST_TOGGLE, N_("Use the Text box font and colors"), P_OFFINT(style_namelistgad),0,0,0},
 	{ST_TOGGLE, N_("Resizable userlist"), P_OFFINT(paned_userlist),0,0,0},
+	{ST_NUMBER, N_("Track away-status on channels smaller than:"), P_OFFINT(away_size_max),0,0,10000},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
@@ -214,6 +216,7 @@ static const setting general_settings[] =
 	{ST_TOGGLE,	N_("Beep on highlighted messages"), P_OFFINT(beephilight), 0, 0, 0},
 	{ST_TOGGLE,	N_("Display MODEs in raw form"), P_OFFINT(raw_modes), 0, 0, 0},
 	{ST_TOGGLE,	N_("Whois on notify"), P_OFFINT(whois_on_notifyonline), N_("Sends a /WHOIS when a user comes online in your notify list"), 0, 0},
+	{ST_TOGGLE,	N_("Hide join/part messages"), P_OFFINT(confmode), N_("Hide channel join/part messages by default"), 0, 0},
 	{ST_NUMBER,	N_("Auto reconnect delay:"), P_OFFINT(recon_delay), 0, 0, 9999},
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -287,14 +290,18 @@ static void
 setup_create_spin (GtkWidget *table, int row, const setting *set)
 {
 	GtkWidget *label, *wid, *rbox, *align;
+	int add = 0;
+
+	if (strlen (set->label) > 30)
+		add = 2;
 
 	label = gtk_label_new (_(set->label));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, row, row + 1,
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1+add, row, row + 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 
 	align = gtk_alignment_new (0.0, 1.0, 0.0, 0.0);
-	gtk_table_attach_defaults (GTK_TABLE (table), align, 1, 2, row, row + 1);
+	gtk_table_attach_defaults (GTK_TABLE (table), align, 1+add, 2+add, row, row + 1);
 
 	rbox = gtk_hbox_new (0, 0);
 	gtk_container_add (GTK_CONTAINER (align), rbox);
@@ -611,6 +618,8 @@ setup_color_ok_cb (GtkWidget *button, GtkWidget *dialog)
 		return;
 	}
 
+	color_change = TRUE;
+
 	gtk_color_selection_get_current_color (GTK_COLOR_SELECTION (cdialog->colorsel), col);
 
 	gdk_colormap_alloc_color (gtk_widget_get_colormap (button), col, TRUE, TRUE);
@@ -662,6 +671,7 @@ setup_create_color_button (GtkWidget *table, int num, int row, int col)
 	else
 		sprintf (buf, "%d", num);
 	but = gtk_button_new_with_label (buf);
+	g_object_set_data (G_OBJECT (but), "xchat-color", 0);
 	gtk_table_attach (GTK_TABLE (table), but, col, col+1, row, row+1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 	g_signal_connect (G_OBJECT (but), "clicked",
@@ -716,6 +726,7 @@ setup_create_color_page (void)
 	setup_create_other_color (_("New Data:"), 20, 10, tab);
 	setup_create_other_color (_("New Message:"), 22, 11, tab);
 	setup_create_other_color (_("Highlight:"), 21, 12, tab);
+	setup_create_other_color (_("Away User:"), 23, 13, tab);
 
 	return box;
 }
@@ -919,6 +930,7 @@ setup_apply (struct xchatprefs *pr)
 	GtkStyle *old_style;
 	int new_pix = FALSE;
 	int noapply = FALSE;
+	int do_ulist = FALSE;
 
 	if (strcmp (pr->background, prefs.background) != 0)
 		new_pix = TRUE;
@@ -934,6 +946,9 @@ setup_apply (struct xchatprefs *pr)
 	if (DIFF (showhostname_in_userlist))
 		noapply = TRUE;
 
+	if (color_change || (DIFF (away_size_max)))
+		do_ulist = TRUE;
+
 	memcpy (&prefs, pr, sizeof (prefs));
 
 #ifdef WIN32
@@ -976,6 +991,9 @@ setup_apply (struct xchatprefs *pr)
 		else
 			log_close (sess);
 
+		if (do_ulist)
+			userlist_rehash (sess);
+
 		list = list->next;
 	}
 
@@ -1076,6 +1094,7 @@ setup_open (void)
 
 	memcpy (&setup_prefs, &prefs, sizeof (prefs));
 
+	color_change = FALSE;
 	setup_window = setup_window_open ();
 
 	g_signal_connect (G_OBJECT (setup_window), "destroy",
