@@ -21,7 +21,6 @@
 
 #include "tabs.h"
 
-#undef ALPHA_SORT
 
 /* ignore "toggled" signal? */
 static int ignore_toggle = FALSE;
@@ -33,6 +32,7 @@ static int ignore_toggle = FALSE;
  *   "c" tab-focus callback function
  *   "foc" currently focused tab
  *   "i" inner hbox (inside the viewport)
+ *   "s" sorted (boolean)
  *   "v" set to 1 if group is vertical type
  *   "b1" first arrow button
  *   "b2" second arrow button
@@ -240,7 +240,7 @@ tab_pressed_cb (GtkToggleButton *tab, GtkWidget *group)
 }
 
 GtkWidget *
-tab_group_new (void *callback, gboolean vertical)
+tab_group_new (void *callback, gboolean vertical, gboolean sorted)
 {
 	GtkWidget *box;
 	GtkWidget *viewport;
@@ -258,6 +258,8 @@ tab_group_new (void *callback, gboolean vertical)
 	gtk_container_set_border_width (GTK_CONTAINER (group), 2);
 	gtk_widget_show (group);
 
+	g_object_set_data (G_OBJECT (group), "s", GINT_TO_POINTER (sorted));
+
 	viewport = gtk_viewport_new (0, 0);
 	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
 	g_signal_connect (G_OBJECT (viewport), "size_request",
@@ -282,7 +284,7 @@ tab_group_new (void *callback, gboolean vertical)
 	g_signal_connect (G_OBJECT (button), "clicked",
 							G_CALLBACK (tab_scroll_right_down_clicked), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
-	gtk_widget_show_all (button);
+	gtk_widget_show (arrow);
 
 	button = gtk_button_new ();
 	g_object_set_data (G_OBJECT (group), "b2", button);
@@ -293,7 +295,7 @@ tab_group_new (void *callback, gboolean vertical)
 	g_signal_connect (G_OBJECT (button), "clicked",
 							G_CALLBACK (tab_scroll_left_up_clicked), group);
 	gtk_box_pack_end (GTK_BOX (group), button, 0, 0, 0);
-	gtk_widget_show_all (button);
+	gtk_widget_show (arrow);
 
 	return group;
 }
@@ -404,8 +406,8 @@ tab_group_switch (GtkWidget *group, int relative, int num)
 {
 	int i, max;
 
-	while (g_main_pending ())
-		g_main_iteration (TRUE);
+	/*while (g_main_pending ())
+		g_main_iteration (TRUE);*/
 
 	tab_group_resize (group);
 
@@ -489,14 +491,24 @@ tab_group_get_focused (GtkWidget *group)
 }
 
 static void
-tab_add_sorted (GtkWidget *box, GtkWidget *tab)
+tab_add_sorted (GtkWidget *group, GtkWidget *box, GtkWidget *tab)
 {
-#ifdef ALPHA_SORT
 	GList *list;
 	GtkBoxChild *child;
-	char *name = GTK_BUTTON (tab)->label_text;
+	char *name;
 	int i = 0;
 
+	if (!GPOINTER_TO_INT (g_object_get_data (G_OBJECT (group), "s")))
+	{
+		gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
+		gtk_widget_show (tab);
+		return;
+	}
+
+	/* sorting TODO:
+    *   - always put servertab first
+    *   - move tab if renamed */
+	name = GTK_BUTTON (tab)->label_text;
 	list = GTK_BOX (box)->children;
 	while (list)
 	{
@@ -514,10 +526,6 @@ tab_add_sorted (GtkWidget *box, GtkWidget *tab)
 		i++;
 		list = list->next;
 	}
-#endif
-
-	gtk_box_pack_start (GTK_BOX (box), tab, 0, 0, 0);
-	gtk_widget_show (tab);
 }
 
 static void
@@ -538,7 +546,7 @@ tab_add_real (GtkWidget *group, GtkWidget *tab, void *family)
 
 		if (g_object_get_data (G_OBJECT (box), "f") == family)
 		{
-			tab_add_sorted (box, tab);
+			tab_add_sorted (group, box, tab);
 			gtk_widget_queue_resize (inner->parent);
 			return;
 		}
@@ -676,12 +684,12 @@ tab_group_add (GtkWidget *group, char *name, void *family, void *userdata,
 		but = gtk_toggle_button_new_with_label (name);
 	gtk_widget_set_name (but, "xchat-tab");
 	/* used to trap right-clicks */
-	g_signal_connect (G_OBJECT (but), "button-press-event",
+	g_signal_connect (G_OBJECT (but), "button_press_event",
 						 	G_CALLBACK (click_cb), userdata);
 	/* avoid prelights */
-	g_signal_connect (G_OBJECT (but), "enter-notify-event",
+	g_signal_connect (G_OBJECT (but), "enter_notify_event",
 						 	G_CALLBACK (tab_ignore_cb), NULL);
-	g_signal_connect (G_OBJECT (but), "leave-notify-event",
+	g_signal_connect (G_OBJECT (but), "leave_notify_event",
 						 	G_CALLBACK (tab_ignore_cb), NULL);
 	g_signal_connect (G_OBJECT (but), "pressed",
 							G_CALLBACK (tab_pressed_cb), group);
@@ -828,7 +836,7 @@ tab_group_set_orientation (GtkWidget *group, gboolean vertical)
 		return group;
 
 	new_group = tab_group_new (g_object_get_data (G_OBJECT (group), "c"),
-								vertical);
+								vertical, FALSE);
 	g_object_set_data (G_OBJECT (new_group), "foc",
 						g_object_get_data (G_OBJECT (group), "foc"));
 	box = g_object_get_data (G_OBJECT (group), "i");
