@@ -20,10 +20,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "../../config.h"
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 #include <stdlib.h>
 
 #include "fe-gtk.h"
@@ -99,7 +95,7 @@ url_treeview_new (GtkWidget *box)
 
 	view = gtkutil_treeview_new (box, GTK_TREE_MODEL (store), NULL,
 	                             URL_COLUMN, _("URL"), -1);
-	g_signal_connect (G_OBJECT (view), "button-press-event",
+	g_signal_connect (G_OBJECT (view), "button_press_event",
 	                  G_CALLBACK (url_treeview_url_clicked_cb), NULL);
 	/* don't want column headers */
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
@@ -169,6 +165,13 @@ fe_url_add (char *urltext)
 	}
 }
 
+static int
+populate_cb (char *urltext, gpointer userdata)
+{
+	fe_url_add (urltext);
+	return TRUE;
+}
+
 void
 url_opengui ()
 {
@@ -202,5 +205,5 @@ url_opengui ()
 
 	gtk_widget_show (urlgrabberwindow);
 
-	tree_foreach (url_tree, (tree_traverse_func *)fe_url_add, NULL);
+	tree_foreach (url_tree, (tree_traverse_func *)populate_cb, NULL);
 }
