@@ -665,6 +665,10 @@ fe_userlist_remove (struct session *sess, struct User *user)
 	return 0;
 }
 void
+fe_userlist_rehash (struct session *sess, struct User *user)
+{
+}
+void
 fe_userlist_move (struct session *sess, struct User *user, int new_row)
 {
 }
@@ -797,11 +801,10 @@ fe_get_int (char *prompt, int def, void *callback, void *ud)
 {
 }
 void
-fe_play_wave (const char *file)
-{
-    play_wave (file);
-}
-void
 fe_idle_add (void *func, void *data)
 {
 }
+void
+fe_ctrl_gui (session *sess, int action)
+{
+}
